﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="CompanyAdminController.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.Admin.CustomerMasterdata
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web.Mvc;

    using Fruitmonitoring.Controllers.General;
    using Fruitmonitoring.Controllers.Interfaces;
    using Fruitmonitoring.Helper.HtmlExtensions.CompanyTypeSelection;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Helper.Utility.GLNChecker;
    using Fruitmonitoring.Infrastructure.ActionAttributes;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;
    using Fruitmonitoring.Models.BindingHandlers;

    using Languages.Emails.Company;

    using Company = Languages.Pages.Admin.CustomerMasterdata.Company.Company;

    #endregion

    /// <summary>
    /// The company admin controller.
    /// </summary>
    [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Admin)]
    public class CompanyAdminController : BaseCompanyController, IDataManipulator<USP_Company_Sel_Result>, IDataManipulator, IExportController
    {
        #region Constructors and Destructors

        #region constructors

        // show all Companies
        /// <summary>
        /// Initializes a new instance of the <see cref="CompanyAdminController"/> class.
        /// </summary>
        public CompanyAdminController()
            : base(
                new CompanyBindingHandlers(new Dictionary<string, object>() { { CompanyBindingHandlers.PROPERTYKEY_COMPANYSTATUSID, null } }),
                GridViewHelper.COMPANYADMINGRIDVIEWID,
                GridViewHelper.GRIDVIEWCOMPONENTSPATH + "Company/_companyGridViewPartial",
                CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Company.Company_id),
                GridViewHelper.COMPANYADMINCONTROLLERNAME,
                new USP_Company_Sel_Result().GetType(),
                true,
                false)
        {
        }
        #endregion

        #endregion

        #region IExportController Members

        #region Public Methods and Operators
        /// <summary>
        /// The export.
        /// </summary>
        /// <param name="exportType">
        /// The export type.
        /// </param>
        /// <param name="columnNames">
        /// The column names.
        /// </param>
        /// <param name="columnTexts">
        /// The column texts.
        /// </param>
        /// <param name="filter">
        /// The filter.
        /// </param>
        /// <param name="sortColumnName">
        /// The sort column name.
        /// </param>
        /// <param name="sortOrder">
        /// The sort order.
        /// </param>
        /// <param name="applySorting">
        /// The apply sorting.
        /// </param>
        /// <param name="applyFilter">
        /// The apply filter.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        public ActionResult Export(
            string exportType,
            string columnNames,
            string columnTexts,
            string filter,
            string sortColumnName,
            string sortOrder,
            bool applySorting,
            bool applyFilter)
        {
            return null;
        }
        #endregion

        #endregion

        #region IDataManipulator<USP_Company_Sel_Result> Member
        /// <summary>
        /// The add.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpGet]
        [AjaxOrChildActionOnly]
        public ActionResult Add()
        {
            return null;
        }

        /// <summary>
        /// The add.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        [AjaxOrChildActionOnly]
        public ActionResult Add(USP_Company_Sel_Result model)
        {
            return null;
        }

        /// <summary>
        /// The edit.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpGet]
        [AjaxOrChildActionOnly]
        public ActionResult Edit(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The edit.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        [AjaxOrChildActionOnly]
        public ActionResult Edit(USP_Company_Sel_Result model)
        {
            return null;
        }

        /// <summary>
        /// The delete.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        [AjaxOrChildActionOnly]
        public ActionResult Delete(Guid id)
        {
            return null;
        }
        #endregion

        #region IDataManipulator Member
        /// <summary>
        /// The check gln.
        /// </summary>
        /// <param name="searchGln">
        /// The search gln.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        public ActionResult CheckGLN(string searchGln)
        {
            return null;
        }

        /// <summary>
        /// The load add and edit view data.
        /// </summary>
        /// <param name="properties">
        /// The properties.
        /// </param>
        [NonAction]
        public void LoadAddAndEditViewData(object properties = null)
        {
        }
        #endregion

        #region private methods
        /// <summary>
        /// Send an Email if the status has switched.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <param name="oldStatus">
        /// The old Status.
        /// </param>
        private void sendInformationIfStatusHasChanged(USP_Company_Sel_Result model, Guid oldStatus)
        {
        }

        /// <summary>
        /// Checks
        /// - if classification has to be set to default
        /// - set CompanyTypes.
        /// </summary>
        /// <param name="model">
        /// Model.
        /// </param>
        /// <param name="companyTypeIds">
        /// New company Ttype Ids.
        /// </param>
        private void prepareValidation(USP_Company_Sel_Result model, IEnumerable<string> companyTypeIds)
        {
        }

        /// <summary>
        /// Set the companyTypeIds in the model.
        /// </summary>
        /// <param name="model">
        /// </param>
        /// <param name="companyTypeIds">
        /// </param>
        [NonAction]
        public void ValidateCompanyTypes(USP_Company_Sel_Result model, IEnumerable<string> companyTypeIds)
        {
        }
        #endregion
    }
}