﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="SampleAdminController.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.Admin.Samples
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web.Mvc;

    using DevExpress.Data.Filtering;
    using DevExpress.Web.Mvc;

    using Fruitmonitoring.Controllers.General;
    using Fruitmonitoring.Controllers.Interfaces;
    using Fruitmonitoring.Helper.Export;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.ActionAttributes;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;
    using Fruitmonitoring.Models.BindingHandlers;

    using Languages;

    #endregion
    
    /// <summary>
    /// The sample admin controller.
    /// </summary>
    [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Admin)]
    public partial class SampleAdminController : BaseSampleDetailController, IExportController, IDataManipulator<SampleAdminModel>
    {
        #region Constructors and Destructors

        #region constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="SampleAdminController"/> class.
        /// </summary>
        public SampleAdminController()
            : base(
                new SampleBindingHandlers(new Dictionary<string, object>() { { SampleBindingHandlers.PROPERTYKEY_USEUSERCOMPANYID, true } }),
                GridViewHelper.SAMPLEADMINGRIDVIEWID,
                GridViewHelper.GRIDVIEWCOMPONENTSPATH + "Sample/_sampleGridViewPartial",
                CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.Sample_id),
                GridViewHelper.SAMPLEADMINCONTROLLERNAME)
        {
            SetAsAdminGridView();
        }
        #endregion

        #endregion

        #region IDataManipulator<SampleAdminModel> Members
        /// <summary>
        /// The add.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        /// <exception cref="NotImplementedException">
        /// </exception>
        public ActionResult Add()
        {
            return null;
        }

        /// <summary>
        /// The add.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        /// <exception cref="NotImplementedException">
        /// </exception>
        public ActionResult Add(SampleAdminModel model)
        {
            return null;
        }

        /// <summary>
        /// The edit.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpGet]
        public ActionResult Edit(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The edit.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(SampleAdminModel model)
        {
            return null;
        }

        /// <summary>
        /// The delete.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        /// <exception cref="NotImplementedException">
        /// </exception>
        public ActionResult Delete(Guid id)
        {
            return null;
        }
        #endregion

        #region Methods

        #region private methods
        /// <summary>
        /// The get fixed filter expression.
        /// </summary>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private string getFixedFilterExpression()
        {
            return null;
        }
        #endregion

        #endregion

        #region Public Methods and Operators

        #region overrides
        /// <summary>
        /// The custom binding core.
        /// </summary>
        /// <param name="gridViewModel">
        /// The grid view model.
        /// </param>
        /// <param name="sortColumn">
        /// The sort column.
        /// </param>
        /// <param name="oldPageSize">
        /// The old page size.
        /// </param>
        /// <returns>
        /// The <see cref="PartialViewResult"/>.
        /// </returns>
        public override PartialViewResult CustomBindingCore(GridViewModel gridViewModel, GridViewColumnState sortColumn = null, int oldPageSize = -1)
        {
            return null;
        }
        #endregion

        #region IDataManipulator Members
        /// <summary>
        /// The load add and edit view data.
        /// </summary>
        /// <param name="properties">
        /// The properties.
        /// </param>
        public void LoadAddAndEditViewData(object properties = null)
        {
        }
        #endregion

        #endregion

        #region actionresults
        /// <summary>
        /// Change the status of the selected samples.
        /// </summary>
        /// <param name="selectedStatus">
        /// New status.
        /// </param>
        /// <param name="sampleIds">
        /// DbGuidHelper.JQUERYLISTSEPARATOR separated list with the guids.
        /// </param>
        /// <param name="commentCancellation">
        /// Required comment for cancellation.
        /// </param>
        /// <returns>
        /// Info messages View.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        [AjaxOrChildActionOnly]
        public ActionResult SampleChangeStatus(string selectedStatus, string sampleIds, string commentCancellation)
        {
            return null;
        }

        /// <summary>
        /// Load the status change content.
        /// </summary>
        /// <returns>
        /// View with the content.
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult LoadChangeStatusPopupContent()
        {
            return null;
        }

        /// <summary>
        /// The export.
        /// </summary>
        /// <param name="exportType">
        /// The export type.
        /// </param>
        /// <param name="columnNames">
        /// The column names.
        /// </param>
        /// <param name="columnTexts">
        /// The column texts.
        /// </param>
        /// <param name="filter">
        /// The filter.
        /// </param>
        /// <param name="sortColumnName">
        /// The sort column name.
        /// </param>
        /// <param name="sortOrder">
        /// The sort order.
        /// </param>
        /// <param name="applySorting">
        /// The apply sorting.
        /// </param>
        /// <param name="applyFilter">
        /// The apply filter.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        public ActionResult Export(
            string exportType,
            string columnNames,
            string columnTexts,
            string filter,
            string sortColumnName,
            string sortOrder,
            bool applySorting,
            bool applyFilter)
        {
            return null;
        }
        #endregion

        #region overrides BaseSampleController
        /// <summary>
        /// The sample detail tab view.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailTabView(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The sample details print view.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailsPrintView(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The sample detail information.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <param name="showAnonymous">
        /// The show anonymous.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailInformation(USP_Sample_Sel_Result model, bool showAnonymous = true)
        {
            return null;
        }

        /// <summary>
        /// The sample detail documents.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailDocuments(List<USP_SampleDocuments_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail documents uploader.
        /// </summary>
        /// <param name="sampleId">
        /// The sample id.
        /// </param>
        /// <param name="companyId">
        /// The company id.
        /// </param>
        /// <param name="showDeleteButton">
        /// The show delete button.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailDocumentsUploader(Guid sampleId, Guid companyId, bool showDeleteButton = false)
        {
            return null;
        }

        /// <summary>
        /// The sample detail certification quality seals.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailCertificationQualitySeals(List<USP_Sample2CertificationQualitySeal_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail certification business.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailCertificationBusiness(List<USP_Sample2CertificationBusiness_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail certification products.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailCertificationProducts(List<USP_Sample2CertificationProduct_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail history.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailHistory(List<USP_SampleHistory_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail results.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailResults(List<USP_SampleResults_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail ratings.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailRatings(List<USP_SampleRating_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail single elements.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailSingleElements(List<USP_SingleElements_Sel_Result> model)
        {
            return null;
        }

        /// <summary>
        /// The sample detail commments.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <param name="sampleId">
        /// The sample id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailCommments(List<USP_SampleComments_Sel_Result> model, Guid sampleId)
        {
            return null;
        }
        #endregion
    }
}