﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="InvestigationOrderController.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.Common
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Web.Mvc;

    using DevExpress.Data.Filtering;
    using DevExpress.Web.Mvc;

    using Fruitmonitoring.Controllers.General;
    using Fruitmonitoring.Controllers.Interfaces;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.ActionAttributes;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;
    using Fruitmonitoring.Models.BindingHandlers;

    using log4net;

    using Languages.Emails;
    using Languages.Emails.InvestigationOrder;

    #endregion

    /// <summary>
    /// The investigation order controller.
    /// </summary>
    [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Orderer)]
    public partial class InvestigationOrderController : BaseInvestigationOrderController, IDataManipulator<USP_Sample_Investigation_Sel_Result>
    {
        #region Constructors and Destructors

        #region constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="InvestigationOrderController"/> class.
        /// </summary>
        public InvestigationOrderController()
            : base(
                new SampleInvestigationOrderBindingHandlers(
                    new Dictionary<string, object>() { { SampleInvestigationOrderBindingHandlers.PROPERTYKEY_USEUSERCOMPANYID, true } }),
                GridViewHelper.INVESTIGATIONORDERCOMMONGRIDVIEWID,
                GridViewHelper.GRIDVIEWCOMPONENTSPATH + "SampleInvestigationOrder/_sampleInvestigationOrderGridViewPartial",
                CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_Investigation.Sample_id),
                GridViewHelper.INVESTIGATIONORDERCOMMONCONTROLLERNAME)
        {
            SetAsAdminGridView();
        }
        #endregion

        #endregion

        #region Public Methods and Operators

        #region overrides
        /// <summary>
        /// The custom binding core.
        /// </summary>
        /// <param name="gridViewModel">
        /// The grid view model.
        /// </param>
        /// <param name="sortColumn">
        /// The sort column.
        /// </param>
        /// <param name="oldPageSize">
        /// The old page size.
        /// </param>
        /// <returns>
        /// The <see cref="PartialViewResult"/>.
        /// </returns>
        [NonAction]
        public override PartialViewResult CustomBindingCore(GridViewModel gridViewModel, GridViewColumnState sortColumn = null, int oldPageSize = -1)
        {
            return null;
        }
        #endregion

        #endregion

        #region Methods
        /// <summary>
        /// The get fixed filter expression.
        /// </summary>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private string getFixedFilterExpression()
        {
            return null;
        }

        /// <summary>
        /// The log send investigation order.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <param name="order">
        /// The order.
        /// </param>
        private void logSendInvestigationOrder(InvestigationOrderModel model, USP_Sample_Investigation_Sel_Result order)
        {
        }

        /// <summary>
        /// The send mail to client.
        /// </summary>
        /// <param name="orderId">
        /// The order id.
        /// </param>
        /// <param name="vDict">
        /// The v dict.
        /// </param>
        /// <param name="clientMailAdresses">
        /// The client mail adresses.
        /// </param>
        /// <param name="ccMailAdresses">
        /// The cc mail adresses.
        /// </param>
        /// <param name="clientLanguage">
        /// The client language.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        private bool sendMailToClient(Guid orderId, ViewDataDictionary vDict, string clientMailAdresses, string ccMailAdresses, string clientLanguage)
        {
            return false;
        }

        /// <summary>
        /// The send mail to laboratory.
        /// </summary>
        /// <param name="orderId">
        /// The order id.
        /// </param>
        /// <param name="vDict">
        /// The v dict.
        /// </param>
        /// <param name="labMailAdresses">
        /// The lab mail adresses.
        /// </param>
        /// <param name="laboratoryLanguage">
        /// The laboratory language.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        private bool sendMailToLaboratory(Guid orderId, ViewDataDictionary vDict, string labMailAdresses, string laboratoryLanguage)
        {
            return false;
        }

        /// <summary>
        /// The send mail to support.
        /// </summary>
        /// <param name="vDict">
        /// The v dict.
        /// </param>
        /// <param name="supportEmailAddress">
        /// The support email address.
        /// </param>
        /// <param name="language">
        /// The language.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        private bool sendMailToSupport(ViewDataDictionary vDict, string supportEmailAddress, string language)
        {
            return false;
        }
        #endregion

        #region IDataManipulator<USP_Sample_Investigation_Sel_Result> Member
        /// <summary>
        /// The add.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpGet]
        [AjaxOrChildActionOnly]
        public ActionResult Add()
        {
            return null;
        }

        /// <summary>
        /// The add.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        [AjaxOrChildActionOnly]
        public ActionResult Add(USP_Sample_Investigation_Sel_Result model)
        {
            return null;
        }

        /// <summary>
        /// The edit.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpGet]
        [AjaxOrChildActionOnly]
        public ActionResult Edit(Guid id)
        {
            return null;
        }

        /// <summary>
        /// If the client (Auftraggeber) is a supplier (Lieferant), the IO's supplier is set to the client.
        /// Also the client (in this case) is always the current user.
        /// </summary>
        /// <param name="model">
        /// </param>
        /// <returns>
        /// The <see cref="USP_Sample_Investigation_Sel_Result"/>.
        /// </returns>
        private USP_Sample_Investigation_Sel_Result setSupplierModelValues(USP_Sample_Investigation_Sel_Result model)
        {
            return null;
        }

        /// <summary>
        /// The edit.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        [AjaxOrChildActionOnly]
        public ActionResult Edit(USP_Sample_Investigation_Sel_Result model)
        {
            return null;
        }

        /// <summary>
        /// The delete.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        /// <exception cref="NotImplementedException">
        /// </exception>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Delete(Guid id)
        {
            return null;
        }
        #endregion

        #region additional Action results
        /// <summary>
        /// The edit temporary.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTemporary(USP_Sample_Investigation_Sel_Result model)
        {
            return null;
        }

        /// <summary>
        /// Shows the popup to accept the agbs.
        /// </summary>
        /// <param name="id">
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpGet]
        [AjaxOrChildActionOnly]
        public ActionResult SendOrder(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The send order.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult SendOrder(InvestigationOrderModel model)
        {
            return null;
        }

        /// <summary>
        /// Generates the cc email string that can be set in email handler.
        /// </summary>
        /// <param name="emailaddresses">
        /// A variable number of parameters containing email address strings.
        /// </param>
        /// <returns>A semicolon separated string with email addresses.</returns>
        private string GenerateCcEmailAddressesForConfirmation(params string[] emailaddresses)
        {
            return null;
        }

        /// <summary>
        /// The show agbs.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpGet]
        [AjaxOrChildActionOnly]
        public ActionResult ShowAGBS()
        {
            return null;
        }

        /// <summary>
        /// Load region selection partial view.
        /// </summary>
        /// <param name="selectedRetailerIds">
        /// Selected retailer guids (separated by DbGuidHelper.DBLISTSEPARATOR).
        /// </param>
        /// <returns>
        /// Partial view.
        /// </returns>
        [HttpPost]
        public ActionResult LoadRegionsData(string selectedRetailerIds = "")
        {
            return null;
        }
        #endregion
    }
}