﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="BaseSampleController.CleanUpModelForForm.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.General
{
    #region usings
    using System;

    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.Security;

    #endregion

    /// <summary>
    /// This class provides functions to remove default values from the sample model
    /// these default values shouldn't be set in the form.
    /// </summary>
    [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Laboratory)]
    public partial class BaseSampleController : BaseGridViewCustomBindingController
    {
        #region methods
        /// <summary>
        /// Sampling date.
        /// </summary>
        /// <param name="samplingDate">
        /// SamplingDate.
        /// </param>
        /// <returns>
        /// The <see cref="DateTime?"/>.
        /// </returns>
        public DateTime? cleanUpSamplingDateForForm(DateTime? samplingDate)
        {
            if (samplingDate == PLACEHOLDERDATE)
            {
                samplingDate = null;
            }

            return samplingDate;
        }

        /// <summary>
        /// Report date.
        /// </summary>
        /// <param name="reportDate">
        /// ReportDate.
        /// </param>
        /// <returns>
        /// The <see cref="DateTime?"/>.
        /// </returns>
        public DateTime? cleanUpReportDateForForm(DateTime? reportDate)
        {
            if (reportDate == PLACEHOLDERDATE)
            {
                reportDate = null;
            }

            return reportDate;
        }

        /// <summary>
        /// Company id.
        /// </summary>
        /// <param name="companyId">
        /// CompanyId.
        /// </param>
        /// <returns>
        /// The <see cref="Guid?"/>.
        /// </returns>
        public Guid? cleanUpCompanyIdForForm(Guid? companyId)
        {
            return null;
        }

        /// <summary>
        /// Company name.
        /// </summary>
        /// <param name="companyName">
        /// CompanyName.
        /// </param>
        /// <param name="companyId">
        /// CompanyId.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string cleanUpCompanyNameForForm(string companyName, Guid? companyId)
        {
            if (companyId == null)
            {
                companyName = string.Empty;
            }

            return companyName;
        }

        /// <summary>
        /// Sampler user id.
        /// </summary>
        /// <param name="samplerUserId">
        /// SamplerUserId.
        /// </param>
        /// <returns>
        /// The <see cref="Guid?"/>.
        /// </returns>
        public Guid? cleanUpSamplerUserIdForForm(Guid? samplerUserId)
        {
            return null;
        }

        /// <summary>
        /// Sampler user first name.
        /// </summary>
        /// <param name="samplerUserId">
        /// SamplerUserId.
        /// </param>
        /// <param name="samplerUserFirstName">
        /// SamplerUserFirstName.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string cleanUpSamplerUserFirstNameForForm(Guid? samplerUserId, string samplerUserFirstName)
        {
            return null;
        }

        /// <summary>
        /// Sampler user last name.
        /// </summary>
        /// <param name="samplerUserId">
        /// SamplerUserId.
        /// </param>
        /// <param name="samplerUserLastName">
        /// SamplerUserLastName.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string cleanUpSamplerUserLastNameForForm(Guid? samplerUserId, string samplerUserLastName)
        {
            return null;
        }

        /// <summary>
        /// Sampler user company name.
        /// </summary>
        /// <param name="samplerUserId">
        /// SamplerUserId.
        /// </param>
        /// <param name="samplerUserCompanyName">
        /// SamplerUserCompanyName.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string cleanUpSamplerUserCompanyNameForForm(Guid? samplerUserId, string samplerUserCompanyName)
        {
            return null;
        }

        /// <summary>
        /// Country.
        /// </summary>
        /// <param name="countryId">
        /// CountryId.
        /// </param>
        /// <returns>
        /// The <see cref="Guid"/>.
        /// </returns>
        public Guid cleanUpCountryForForm(Guid countryId)
        {
            return new Guid();
        }

        /// <summary>
        /// Farming type.
        /// </summary>
        /// <param name="farmingType">
        /// FarmingType.
        /// </param>
        /// <returns>
        /// The <see cref="Guid"/>.
        /// </returns>
        public Guid cleanUpFarmingTypeForForm(Guid farmingType)
        {
            return new Guid();
        }

        /// <summary>
        /// Offer form.
        /// </summary>
        /// <param name="offerForm">
        /// OfferForm.
        /// </param>
        /// <returns>
        /// The <see cref="Guid"/>.
        /// </returns>
        public Guid cleanUpOfferFormForForm(Guid offerForm)
        {
            return new Guid();
        }

        /// <summary>
        /// BioProductType.
        /// </summary>
        /// <param name="bioProductType">
        /// BioProductType.
        /// </param>
        /// <returns>
        /// The <see cref="Guid"/>.
        /// </returns>
        public Guid cleanUpBioProductTypeForForm(Guid bioProductType)
        {
            return new Guid();
        }

        /// <summary>
        /// SampleType.
        /// </summary>
        /// <param name="sampleType">
        /// SampleType.
        /// </param>
        /// <returns>
        /// The <see cref="Guid"/>.
        /// </returns>
        public Guid cleanUpSampleTypeForForm(Guid sampleType)
        {
            return new Guid();
        }

        /// <summary>
        /// SampleKind.
        /// </summary>
        /// <param name="sampleKind">
        /// SampleKind.
        /// </param>
        /// <returns>
        /// The <see cref="Guid"/>.
        /// </returns>
        public Guid cleanUpSampleKindForForm(Guid sampleKind)
        {
            return new Guid();
        }

        /// <summary>
        /// BrandType.
        /// </summary>
        /// <param name="brandTypeId">
        /// BrandTypeId.
        /// </param>
        /// <returns>
        /// The <see cref="Guid?"/>.
        /// </returns>
        public Guid? cleanUpBrandTypeForForm(Guid? brandTypeId)
        {
            return new Guid();
        }
        #endregion
    }
}