﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="IDataManipulator.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.Interfaces
{
    /// <summary>
    /// The DataManipulator interface.
    /// </summary>
    public interface IDataManipulator
    {
        #region Public Methods and Operators
        /// <summary>
        /// Loads all needed ViewData for the edit and add form.
        /// </summary>
        /// <param name="properties">
        /// Additional parameters for viewdata.
        /// </param>
        void LoadAddAndEditViewData(object properties = null);
        #endregion
    }
}

namespace Fruitmonitoring.Controllers.Interfaces
{
    #region usings
    using System;
    using System.Web.Mvc;

    #endregion

    /// <summary>
    /// The DataManipulator interface.
    /// </summary>
    /// <typeparam name="T">
    /// </typeparam>
    public interface IDataManipulator<T> : IDataManipulator
    {
        #region Public Methods and Operators
        /// <summary>
        /// Shows the add form.
        /// </summary>
        /// <returns>
        /// View for add form.
        /// </returns>
        [HttpGet]
        ActionResult Add();

        /// <summary>
        /// The add.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        ActionResult Add(T model);

        /// <summary>
        /// Delete an entry from database.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// Redirect view.
        /// </returns>
        [HttpPost]
        ActionResult Delete(Guid id);

        /// <summary>
        /// Shows the edit form.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// View for edit form.
        /// </returns>
        [HttpGet]
        ActionResult Edit(Guid id);

        /// <summary>
        /// Edit an entry from database.
        /// </summary>
        /// <typeparam name="T">
        /// Model type.
        /// </typeparam>
        /// <param name="model">
        /// Model.
        /// </param>
        /// <returns>
        /// Redirect view.
        /// </returns>
        [HttpPost]
        ActionResult Edit(T model);
        #endregion
    }
}