#region Copyright  convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="ParameterGroupLaboratoryController.cs.cs" company="convivo GmbH">
//    Copyright  2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.Laboratory.Samples
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Collections.Specialized;
    using System.Configuration;
    using System.Diagnostics;
    using System.IO;
    using System.Linq;
    using System.Text;
    using System.Text.RegularExpressions;
    using System.Web;
    using System.Web.Mvc;
    using System.Web.Routing;

    using DevExpress.Web;
    using DevExpress.Web.Mvc;

    using Excel;

    using Fruitmonitoring.Controllers.General;
    using Fruitmonitoring.Controllers.Interfaces;
    using Fruitmonitoring.Helper.Export;
    using Fruitmonitoring.Helper.ParameterGroup;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;
    using Fruitmonitoring.Models.BindingHandlers;

    using Languages;
    using Languages.Components;

    using Global = Languages.Emails.Global;
    using Parameter = Languages.ModelData.Parameter.Parameter;
    using ParameterGroup = Languages.Pages.Laboratory.Samples.ParameterGroup.ParameterGroup;

    #endregion

    /// <summary>
    /// The parameter group laboratory controller.
    /// </summary>
    [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Laboratory)]
    public class ParameterGroupLaboratoryController : BaseParameterGroupController, IExportController, IDataManipulator<USP_ParameterGroup_Sel_Result>
    {
        #region ImportState enum

        #region Enums
        /// <summary>
        /// The import state.
        /// </summary>
        public enum ImportState
        {
            /// <summary>
            /// The failed.
            /// </summary>
            FAILED,

            /// <summary>
            /// The succeed.
            /// </summary>
            SUCCEED,

            /// <summary>
            /// The failedvalidation.
            /// </summary>
            FAILEDVALIDATION
        }
        #endregion

        #endregion

        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="ParameterGroupLaboratoryController"/> class.
        /// </summary>
        public ParameterGroupLaboratoryController()
            : base(
                new ParameterGroupBindingHandlers(new Dictionary<string, object>() { { ParameterGroupBindingHandlers.PROPERTYKEY_USEUSERCOMPANYID, true } }),
                GridViewHelper.PARAMETERGROUPLABORATORYGRIDVIEWID,
                GridViewHelper.GRIDVIEWCOMPONENTSPATH + "ParameterGroup/_parameterGroupLaboratoryGridViewPartial",
                CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_ParameterGroup.ParameterGroup_id),
                GridViewHelper.PARAMETERGROUPLABORATORYCONTROLLERNAME)
        {
        }
        #endregion

        #region Public Methods and Operators

        #region IExportController Member
        /// <summary>
        /// The export.
        /// </summary>
        /// <param name="exportType">
        /// The export type.
        /// </param>
        /// <param name="columnNames">
        /// The column names.
        /// </param>
        /// <param name="columnTexts">
        /// The column texts.
        /// </param>
        /// <param name="filter">
        /// The filter.
        /// </param>
        /// <param name="sortColumnName">
        /// The sort column name.
        /// </param>
        /// <param name="sortOrder">
        /// The sort order.
        /// </param>
        /// <param name="applySorting">
        /// The apply sorting.
        /// </param>
        /// <param name="applyFilter">
        /// The apply filter.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        public ActionResult Export(
            string exportType,
            string columnNames,
            string columnTexts,
            string filter,
            string sortColumnName,
            string sortOrder,
            bool applySorting,
            bool applyFilter)
        {
            // get data
            var parameterGroupBindingHandler = new ParameterGroupBindingHandlers(
                applySorting ? sortColumnName : null,
                applySorting ? parseSortOrder(sortOrder) : BaseBindingHandlers.DbOrderType.NONE,
                0,
                Exporter.MAXROWS,
                applyFilter ? GridViewFilterHelper.GenerateXmlFilter(filter, new USP_ParameterGroup_Sel_Result().GetType()) : null,
                null,
                new Dictionary<string, object>() { { ParameterGroupBindingHandlers.PROPERTYKEY_USEUSERCOMPANYID, true } });
            parameterGroupBindingHandler.SetMembershipUser(User);
            IQueryable<USP_ParameterGroup_Sel_Result> model = parameterGroupBindingHandler.Model.AsQueryable<USP_ParameterGroup_Sel_Result>();

            // export
            return Export<USP_ParameterGroup_Sel_Result>(model, exportType, columnNames, columnTexts);
        }
        #endregion

        #region IDataManipulator Member
        /// <summary>
        /// The load add and edit view data.
        /// </summary>
        /// <param name="properties">
        /// The properties.
        /// </param>
        public void LoadAddAndEditViewData(object properties = null)
        {
            var parameterGroupId = Guid.Parse(properties.GetType().GetProperty("ParameterGroupId").GetValue(properties).ToString());

            ViewData.Add("ParameterList", Db.USP_Parameter_Sel(null, parameterGroupId, true, null, null, null, 0, null, null, CultureHelper.GetCultureISO639_3()).ToList());
            ViewData.Add(
                "ParameterTypes",
                Db.USP_ParameterType_Sel(null, true, 0, null, CultureHelper.GetCultureISO639_3()).ToList<USP_ParameterType_Sel_Result>());
        }
        #endregion

        #endregion

        #region IDataManipulator<USP_ParameterGroup_Sel_Result> Member
        /// <summary>
        /// The add.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        /// <exception cref="NotImplementedException">
        /// </exception>
        [HttpGet]
        public ActionResult Add()
        {
            return null;
        }

        /// <summary>
        /// The add.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Add(USP_ParameterGroup_Sel_Result model)
        {
            return null;
        }

        /// <summary>
        /// The edit.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpGet]
        public ActionResult Edit(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The edit.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        [ValidateInput(false)] // deactivate automatic request verification. It is done manually!
        public ActionResult Edit(USP_ParameterGroup_Sel_Result model)
        {
            return null;
        }

        /// <summary>
        /// The delete.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Delete(Guid id)
        {
            return null;
        }
        #endregion

        #region overrides
        /// <summary>
        /// The parameter group detail.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        public override ActionResult ParameterGroupDetail(Guid id)
        {
            return null;
        }

        /// <summary>
        /// The index.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        public override ActionResult Index()
        {
            return null;
        }
        #endregion

        #region methods
        /// <summary>
        /// Builds the xml string which holds all selected parameter.
        /// </summary>
        /// <param name="parameterGroupItems">
        /// The parameter Group Items.
        /// </param>
        /// <returns>
        /// Xml for the stored procedure.
        /// </returns>
        private static string generateParameterGroupXml(List<ParameterGroupItem> parameterGroupItems)
        {
            return null;
        }

        /// <summary>
        /// Generates a list of all parametergroupitems that should be included to the xml for the SP.
        /// </summary>
        /// <param name="form">
        /// Current form collection ("Request.Form").
        /// </param>
        /// <returns>
        /// Lsit which contains all parametergroupitems for the xml.
        /// </returns>
        private List<ParameterGroupItem> getParameterGroupitemList(NameValueCollection form)
        {
            return null;
        }
        #endregion

        #region ActionResults
        /// <summary>
        /// Export details of a single parametergroup to excel
        /// the exported file can used by the laboratories to edit an scope of investigation and import the changes.
        /// </summary>
        /// <param name="parameterGroupId">
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        public ActionResult ExportSingle(Guid parameterGroupId)
        {
            return null;
        }

        /// <summary>
        /// Set import validation settings.
        /// </summary>
        public UploadControlValidationSettings ValidationSettings
        {
            get
            {
                return new UploadControlValidationSettings()
                       {
                           AllowedFileExtensions = new[] { ".xlsx" },
                           MaxFileSize = long.Parse(ConfigurationManager.AppSettings["MaxFileUploadSize"].ToString()) * 1024 * 1024,
                           MaxFileSizeErrorText = DocumentFileUploader.MaxFileSizeErrorText,
                           NotAllowedFileExtensionErrorText = DocumentFileUploader.NotAllowedFileExtensionErrorText,
                           GeneralErrorText = DocumentFileUploader.FileUploadGeneralError
                       };
            }

            private set
            {
            }
        }

        /// <summary>
        /// Nothing to do here
        /// the fileuploadcomplete handler is added in "initialize" function.
        /// </summary>
        /// <param name="ucInvestigationOrderImportSelection">
        /// The uc Investigation Order Import Selection.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        public ActionResult FileUpload(IEnumerable<UploadedFile> ucInvestigationOrderImportSelection)
        {
            return null;
        }

        /// <summary>
        /// The initialize.
        /// </summary>
        /// <param name="requestContext">
        /// The request context.
        /// </param>
        protected override void Initialize(RequestContext requestContext)
        {
        }

        /// <summary>
        /// </summary>
        /// <param name="sender">
        /// </param>
        /// <param name="e">
        /// </param>
        /// <param name="companyId">
        /// The company Id.
        /// </param>
        private static void FileUploadCompleteHandler(object sender, FileUploadCompleteEventArgs e, Guid companyId)
        {
        }

        /// <summary>
        /// </summary>
        /// <param name="file">
        /// </param>
        /// <param name="companyId">
        /// The company Id.
        /// </param>
        /// <returns>
        /// The <see cref="ImportState"/>.
        /// </returns>
        private static ImportState StartImport(UploadedFile file, Guid companyId)
        {
            return new ImportState();
        }

        /// <summary>
        /// Set status of the parametergroup to "wait for approval".
        /// </summary>
        /// <param name="parameterGroupId">
        /// Parameter group id.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult ParameterGroupRelease(Guid parameterGroupId)
        {
            return null;
        }

        /// <summary>
        /// Copies a parameter group.
        /// </summary>
        /// <param name="parameterGroupId">
        /// Parameter group that should copied.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CopyParameterGroup(Guid parameterGroupId)
        {
            return null;
        }
        #endregion
    }
}