﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="SampleLaboratoryBaseController.Validation.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.Laboratory.Samples
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web.Mvc;

    using DevExpress.Data.Filtering;

    using Fruitmonitoring.Controllers.General;
    using Fruitmonitoring.Helper.HtmlExtensions.DropDownEditSelection;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Helper.Validation;
    using Fruitmonitoring.Helper.Validation.Interfaces;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;

    using Languages;

    using Sample = Languages.ModelData.Sample.Sample;

    #endregion

    [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Laboratory)]
    public partial class SampleLaboratoryBaseController : BaseSampleDetailController
    {
        #region Properties

        #region Fields
        readonly ISampleValidationHelper helper = new SampleValidationHelper();
        #endregion

        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// additional validation for model properties that depend on other properties
        /// </summary>
        /// <param name="model">model that should be validated</param>
        /// <param name="viewData">current viewdata dictionary</param>
        /// <param name="retailerCheckBoxArrayIds">all selected retailer ids</param>
        /// <param name="validateForShortForm">if the model comes from the short-form, set this to true, otherwise false</param>
        /// <returns></returns>
        public bool additionalDependencyValidation(
            USP_Sample_Sel_Result model,
            ViewDataDictionary viewData,
            string[] retailerCheckBoxArrayIds,
            string[] certificationBusinessSelectionIds,
            string[] certificationProductSelectionIds,
            string[] certificationQualitySealSelectionIds,
            Guid companyId,
            Guid? accountId,
            bool validateForShortForm = false,
            bool isXMLschnittstelleSample = false)
        {
            return false;
        }

        /// <summary>
        /// do additional validation for temporary save here
        /// </summary>
        /// <param name="model">sample model</param>
        /// <param name="retailerSelectionIds">The selected retailer ids.</param>
        /// <param name="accountId">The useres account id.</param>
        public void AdditionalTemporaryValidation(USP_Sample_Sel_Result model, string[] retailerSelectionIds, Guid accountId)
        {
        }

        /// <summary>
        /// Get existing sample from db.
        /// </summary>
        /// <param name="accountId">The account id.</param>
        /// <param name="sampleId">The sample id.</param>
        /// <param name="companyLabId">The laboratory id.</param>
        /// <returns>The sample object from db or null if not exists (or not authorized).</returns>
        public USP_Sample_Sel_Result getSavedSample(Guid? accountId, Guid? sampleId, Guid? companyLabId)
        {
            return new USP_Sample_Sel_Result();
        }

        /// <summary>
        /// Can the user access the sample?
        /// </summary>
        /// <param name="savedSample">The sample saved in db.</param>
        /// <returns></returns>
        public bool isAuthorizedForSample(USP_Sample_Sel_Result savedSample)
        {
            return false;
        }

        public bool validateReadOnlyFields(USP_Sample_Sel_Result model, USP_Sample_Sel_Result savedSample, bool isValid)
        {
            return false;
        }
        #endregion

        #region Methods
        /// <summary>
        /// examined product
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        protected bool validateExaminedProduct(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        protected bool validateExaminedProductGuidNone(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// returns true if Region depence on Retailer is correct, otherwise ModelError is added.
        /// </summary>
        /// <param name="model"></param>
        /// <param name="p"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        protected bool validateRegion(USP_Sample_Sel_Result model, string[] retailerIds, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate report date
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        protected bool validateReportDate(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate retailer
        /// </summary>
        /// <param name="model"></param>
        /// <param name="retailerCheckBoxArrayIds"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        protected bool validateRetailer(USP_Sample_Sel_Result model, string[] retailerCheckBoxArrayIds, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate sample examination
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        protected bool validateSampleExamination(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate sample kind
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        protected bool validateSampleKind(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate sample rating
        ///
        /// each sample must provide an "FM" Rating
        /// if an private brand is selected, a special Rating must provided to, if the company has an special schema
        ///
        /// info: validateBrand() must called before to get a proper validation
        ///
        /// info: complex code :(
        /// </summary>
        /// <param name="model"></param>
        /// <param name="retailerCheckBoxArrayIds"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        protected bool validateSampleRating(USP_Sample_Sel_Result model, string[] retailerCheckBoxArrayIds, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate that the sampleStatus is "in progress"
        /// otherwise the sample can't be edited
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        protected bool validateSampleStatusSampleInProgress(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate sampling date
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        protected bool validateSamplingDateBeforeLabDate(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate that there is at least one scope of investigation that has the status "Approval"
        /// </summary>
        /// <param name="isValid"></param>
        /// <returns></returns>
        protected bool validateScopeOfInvestigation(bool isValid, Guid companyId)
        {
            return false;
        }

        /// <summary>
        /// returns true if there is at least 1 attachment added for the sample
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateAttachment(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// returns true if bioInstance depence on BioProductType_id is correct, otherwise ModelError is added.
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateBioInstance(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// returns true if brandtype depence on BrandType_id, PrivateBrand_Secured_id, Brand and retailer is correct, otherwise ModelError is added.
        /// If PrivateBrand_Secured_id is disallowed for given brandtypeid the value is set to default.
        /// </summary>
        /// <param name="model"></param>
        /// <param name="retailer"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateBrand(USP_Sample_Sel_Result model, string[] retailer, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// returns true if at least one Certification Business is selected, otherwise ModelError is added.
        /// </summary>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateCertificationBusiness(string[] certificationBusinessSelectionIds, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// returns true if at least one product certification is selected, otherwise ModelError is added.        
        /// </summary>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateCertificationProducts(string[] certificationProductSelectionIds, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// returns true if at least one quality seal certification is selected, otherwise ModelError is added.        
        /// </summary>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateCertificationQualitySeals(string[] certificationQualitySealSelectionIds, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate that the sampleStatus is "order edit"
        /// otherwise the order/sample can't be edited and ModelError is added
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateFarmingType(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// Returns isValid if at least one the three order types is true, otherwise returns false.        
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateInvestigationOrderExamination(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// Validates private brand.
        /// </summary>
        /// <param name="model">Sample model.</param>
        /// <param name="retailer">Selected retailers.</param>
        /// <param name="isValid">Current isValid state.</param>
        /// <returns>Rturns true, if private brand selection is valid. Otherwise false.</returns>
        private bool validatePrivateBrand(USP_Sample_Sel_Result model, string[] retailer, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate company factory is not null or "---" (default company)
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateSampleCompanyFactory(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate company laboratory is not null or "---" (default company)
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateSampleCompanyLaboratory(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate company producer
        /// producer is only required, if the company-client contains the companytype "producer"
        /// If its not required set defaultCompanyNone
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateSampleCompanyProducer(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate company sender is not null or "---" (default company)
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateSampleCompanySender(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate company Supplier is not null or "---" (default company)
        /// </summary>
        /// <param name="model"></param>
        /// <param name="retailerCheckBoxArrayIds"></param>
        /// <param name="allowComanyNone">Is company allowed in this validation?</param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateSampleCompanySupplier(USP_Sample_Sel_Result model, string[] retailerCheckBoxArrayIds, bool allowComanyNone, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// Do server-side validation for SampleQuantity and SampleCount, at least one of them has to be specified.
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateSampleQuantityAndCount(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate samplerUser is not null or "---" (default User)
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateSampleSamplerUser(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// returns true if Samplingdate is correct, otherwise ModelError is added.
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isValid"></param>
        /// <returns></returns>
        private bool validateSamplingDateNotInFuture(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate processing status is not null or "---" (default processing status)
        /// </summary>
        /// <param name="model">The model</param>
        /// <param name="isValid">Is valid</param>
        /// <returns></returns>
        private bool validateProcessingStatus(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        /// <summary>
        /// validate intended use is not null or "---" (default intended use)
        /// </summary>
        /// <param name="model">The model</param>
        /// <param name="isValid">Is valid</param>
        /// <returns></returns>
        private bool validateIntendedUse(USP_Sample_Sel_Result model, bool isValid)
        {
            return false;
        }

        #endregion
    }
}