﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="SampleLaboratoryBaseController.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.Laboratory.Samples
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Configuration;
    using System.Linq;
    using System.Web.Mvc;
    using System.Web.Routing;

    using DevExpress.Data.Filtering;
    using DevExpress.Web.Mvc;

    using Fruitmonitoring.Controllers.General;
    using Fruitmonitoring.Controllers.Interfaces;
    using Fruitmonitoring.Helper.Export;
    using Fruitmonitoring.Helper.HtmlExtensions.SampleRatingManagement;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.AccessRules;
    using Fruitmonitoring.Infrastructure.ActionAttributes;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;
    using Fruitmonitoring.Models.BindingHandlers;

    using Languages.Emails;
    using Languages.Emails.Samples;

    using Sample = Languages.Pages.Laboratory.Samples.Samples.Sample;

    #endregion

    [CustomAuthorize(AppRole = FruitmonitoringRoles.SuperAdmin | FruitmonitoringRoles.Admin | FruitmonitoringRoles.Laboratory)]
    public partial class SampleLaboratoryBaseController : BaseSampleDetailController, IDataManipulator<USP_Sample_Sel_Result>, IExportController
    {
        #region SAMPLEAUTOCOPY enum

        #region Enums
        public enum SAMPLEAUTOCOPY
        {
            CANCOPY,

            CANNOTCOPY
        }
        #endregion

        #endregion

        #region Properties

        #region Constants
        public const int BUSINESSDAYSTOADD = 1;
        #endregion

        #region Fields
        public ReadOnlyCollection<DayOfWeek> NONBUSINESSDAYS = new List<DayOfWeek> { DayOfWeek.Sunday }.AsReadOnly();
        #endregion

        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// add all PSM not detectable ratings to a sample
        /// </summary>
        /// <param name="sampleId"></param>
        /// <returns></returns>
        [ValidateAntiForgeryToken]
        [AjaxOrChildActionOnly]
        public ActionResult AddDefaultExaminationStandardRating(Guid sampleId)
        {
            return null;
        }

        /// <summary>
        /// copies a complete sample to a new sample
        /// </summary>
        /// <param name="sampleId">the id of the sample that should copied</param>
        /// <returns></returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CopyCompleteSample(Guid sampleId)
        {
            return null;
        }

        /// <summary>
        /// copies a complete sample within a sample
        /// </summary>
        /// <param name="sampleId">the id of the sample that should copied</param>
        /// <returns></returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CopyCompleteSampleWithinASample(Guid sampleId)
        {
            return null;
        }

        #region overrides
        public override PartialViewResult CustomBindingCore(GridViewModel gridViewModel, GridViewColumnState sortColumn = null, int oldPageSize = -1)
        {
            return null;
        }
        #endregion

        /// <summary>
        /// load examined products view
        /// </summary>
        /// <param name="sampleId">sample id</param>
        /// <returns>partial view</returns>
        public ActionResult LoadExaminedProductsData(Guid sampleId)
        {
            return null;
        }

        /// <summary>
        /// load region selection partial view
        /// </summary>
        /// <param name="selectedRetailerIds">selected retailer guids (separated by DbGuidHelper.JQUERYLISTSEPARATOR)</param>
        /// <returns>partial view</returns>
        public ActionResult LoadRegionsData(string selectedRetailerIds = "")
        {
            return null;
        }
        #endregion

        #region Methods
        protected virtual string getFixedFilterExpression()
        {
            return null;
        }

        /// <summary>
        /// get a list of ratings for PSM
        /// </summary>
        /// <returns></returns>
        private List<USP_RatingArea_Sel_Result> getDefaultExaminationAutoRatingAreas()
        {
            return null;
        }
        #endregion

        #region constructor
        public SampleLaboratoryBaseController()
            : base(new SampleBindingHandlers(
                new Dictionary<string, object>() { { SampleBindingHandlers.PROPERTYKEY_USEUSERCOMPANYID, true } }
                ),
                GridViewHelper.SAMPLELABORATORYGRIDVIEWID,
                GridViewHelper.GRIDVIEWCOMPONENTSPATH + "Sample/_sampleLaboratoryGridViewPartial",
                CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.Sample_id),
                GridViewHelper.SAMPLELABORATORYBASECONTROLLERNAME)
        {
            SetAsAdminGridView();
        }

        protected SampleLaboratoryBaseController(
            SampleBindingHandlers sampleBindingHandlers,
            string gridviewId,
            string gridviewPartialPath,
            string keyFieldName,
            string controllerName)
            : base(sampleBindingHandlers, gridviewId, gridviewPartialPath, keyFieldName, controllerName)
        {
        }
        #endregion

        #region actionresults
        /// <summary>
        /// before action is executed
        /// </summary>
        /// <param name="requestContext"></param>
        protected override void OnActionExecuting(ActionExecutingContext filterContext)
        {
        }

        /// <summary>
        /// set status of a sample to "transferred"
        /// </summary>
        /// <param name="sampleIds">the sample ids</param>
        /// <returns></returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult TransferSample(string sampleIds)
        {
            return null;
        }

        /// <summary>
        /// If transfer of samples was successfull, write to sample.log.
        /// </summary>
        /// <param name="samples"></param>
        /// <param name="newStatus"></param>
        private void logSuccessfulSampleTransfer(List<USP_Sample_Sel_Result> samples, Guid newStatus)
        {
        }

        /// <summary>
        /// check if there can be added copies of the sample automatically
        /// </summary>
        /// <param name="sampleId">sample Id</param>
        /// <returns></returns>
        [HttpPost]
        [AjaxOrChildActionOnly]
        public ActionResult EditSampleAndCheckCopy(Guid sampleId)
        {
            return null;
        }

        [HttpPost]
        public ActionResult Export(
            string exportType,
            string columnNames,
            string columnTexts,
            string filter,
            string sortColumnName,
            string sortOrder,
            bool applySorting,
            bool applyFilter)
        {
            return null;
        }

        /// <summary>
        /// checks if there can created copies of an sample automatically
        /// </summary>
        /// <param name="sampleId"></param>
        /// <returns></returns>
        private bool canAutoCopySample(Guid sampleId)
        {
            return false;
        }

        /// <summary>
        /// add copies for a sample if the sample is saved and single elements of the sample should be investigated
        /// </summary>
        /// <param name="sampleId">sample id</param>
        /// <returns></returns>
        [HttpPost]
        [AjaxOrChildActionOnly]
        public ActionResult EditSampleAndAddCopies(Guid sampleId)
        {
            return null;
        }

        /// <summary>
        /// load the sample cancellation content
        /// </summary>
        /// <returns>view with the content</returns>
        [AjaxOrChildActionOnly]
        public ActionResult LoadSampleCancellationPopupContent()
        {
            return null;
        }

        /// <summary>
        /// change the status of the selected samples to cancellation
        /// </summary>
        /// <param name="sampleIds">DbGuidHelper.JQUERYLISTSEPARATOR separated list with the guids</param>
        /// <param name="commentCancellation">required comment for cancellation</param>
        /// <returns>view</returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        [AjaxOrChildActionOnly]
        public ActionResult DoSampleCancellation(string sampleIds, string commentCancellation)
        {
            return null;
        }
        #endregion

        #region IDataManipulator<USP_Sample_Sel_Result> Member
        /// <summary>
        /// get the sample-add form
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        [AjaxOrChildActionOnly]
        public ActionResult Add()
        {
            return null;
        }

        /// <summary>
        /// prepare sample model for saving to db
        /// </summary>
        /// <param name="model">sample model</param>
        /// <returns></returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        [AjaxOrChildActionOnly]
        public ActionResult Add(USP_Sample_Sel_Result model)
        {
            return null;
        }

        /// <summary>
        /// save a new sample to db
        /// </summary>
        /// <param name="model">sample model</param>
        /// <param name="retailerSelectionIds">retailer ids</param>
        /// <param name="certificationBusinessSelectionIds">certification business ids</param>
        /// <param name="certificationProductSelectionIds">certification product ids</param>
        /// <param name="certificationQualitySealSelectionIds">certification qualilty seal ids</param>
        /// <returns>partial view with notification. if failed, status code 400 is returned</returns>
        [HttpPost]
        private ActionResult AddSample(
            USP_Sample_Sel_Result model,
            string[] retailerSelectionIds,
            string[] certificationBusinessSelectionIds,
            string[] certificationProductSelectionIds,
            string[] certificationQualitySealSelectionIds,
            bool addShortSample = false)
        {
            return null;
        }

        /// <summary>
        /// IsSavedByUser of saved sample
        /// </summary>
        /// <param name="sampleId">Id of the sample</param>
        /// <param name="accountGuid">Id of the user.</param>
        /// <param name="companyGuid">Id of the company of the user.</param>
        /// <returns>Value of IsSavedByUser</returns>
        public bool IsSavedByUserStatus(Guid sampleId, Guid accountGuid, Guid companyGuid)
        {
            return false;
        }

        /// <summary>
        /// Send emails if a sample has been locked
        /// </summary>
        /// <param name="model"></param>
        /// <param name="retailerSelectionIds"></param>
        private void sendSampleLockedEmails(USP_Sample_Sel_Result model, string[] retailerSelectionIds, Guid accountId, bool isXmlWebservice)
        {
        }

        /// <summary>
        /// If the creation date of a sample is too far away from the lab entry date, the sample should be locked, see FRUITDB-798
        /// </summary>
        /// <param name="model"></param>
        public void HandleSampleLocking(
            USP_Sample_Sel_Result model,
            string[] retailerSelectionIds,
            Guid accountId,
            Guid companyId,
            bool isXmlWebservice,
            bool isSavedByUser)
        {
        }

        /// <summary>
        /// Checks whether the lab entry date of a sample is considerably smaller than the creation date of the record.
        /// Uses the date part only, not the time of day.
        /// </summary>
        /// <param name="model"></param>
        /// <param name="isSavedByUser"></param>
        /// <returns></returns>
        private bool isSampleTooOld(USP_Sample_Sel_Result model)
        {
            return false;
        }

        /// <summary>
        /// get information that is needed to load the "edit-add" viewdata
        /// </summary>
        /// <param name="model">current sample model</param>
        /// <returns></returns>
        private object getAdditionalLoadAndEditViewDataProperties(USP_Sample_Sel_Result model, bool isSampleRegistration)
        {
            return null;
        }

        /// <summary>
        /// temporary save 
        /// values are not validated and null values are replaces with placeholders
        /// </summary>
        /// <param name="model">sample model</param>
        /// <returns></returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTemporary(USP_Sample_Sel_Result model)
        {
            return null;
        }

        /// <summary>
        /// get the sample-edit form
        /// </summary>
        /// <param name="retailerCompanyId"></param>
        /// <returns></returns>
        [HttpGet]
        [AjaxOrChildActionOnly]
        public ActionResult Edit(Guid id)
        {
            return null;
        }

        /// <summary>
        /// sample form request comes in here
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [HttpPost]
        [ValidateAntiForgeryToken]
        [AjaxOrChildActionOnly]
        public ActionResult Edit(USP_Sample_Sel_Result model)
        {
            return null;
        }

        /// <summary>
        /// save the edited sample to db
        /// </summary>
        /// <param name="model">sample model</param>
        /// <param name="retailerSelectionIds">retailer ids</param>
        /// <param name="certificationBusinessSelectionIds">certification business ids</param>
        /// <param name="certificationProductSelectionIds">certification product ids</param>
        /// <param name="certificationQualitySealSelectionIds">certification qualilty seal ids</param>
        /// <returns>partial view with notification. if failed, status code 400 is returned</returns>
        [HttpPost]
        public ActionResult EditSample(
            USP_Sample_Sel_Result model,
            string[] retailerSelectionIds,
            string[] certificationBusinessSelectionIds,
            string[] certificationProductSelectionIds,
            string[] certificationQualitySealSelectionIds)
        {
            return null;
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Delete(Guid id)
        {
            return null;
        }
        #endregion

        #region overrides BaseSampleController
        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailTabView(Guid id)
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailsPrintView(Guid id)
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailInformation(USP_Sample_Sel_Result model, bool showAnonymous = true)
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailDocuments(List<USP_SampleDocuments_Sel_Result> model)
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailDocumentsUploader(Guid sampleId, Guid companyId, bool showDeleteButton = false)
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailCertificationQualitySeals(List<USP_Sample2CertificationQualitySeal_Sel_Result> model)
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailCertificationBusiness(List<USP_Sample2CertificationBusiness_Sel_Result> model)
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailCertificationProducts(List<USP_Sample2CertificationProduct_Sel_Result> model)
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailHistory(List<USP_SampleHistory_Sel_Result> model)
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailResults(List<USP_SampleResults_Sel_Result> model)
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailRatings(List<USP_SampleRating_Sel_Result> model)
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailSingleElements(List<USP_SingleElements_Sel_Result> model)
        {
            return null;
        }

        [AjaxOrChildActionOnly]
        public override ActionResult SampleDetailCommments(List<USP_SampleComments_Sel_Result> model, Guid sampleId)
        {
            return null;
        }
        #endregion
    }
}