﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="Exporter.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Export
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Data.Entity.Core.Objects;
    using System.Drawing;
    using System.Globalization;
    using System.IO;
    using System.Linq;
    using System.Text.RegularExpressions;
    using System.Web;
    using System.Web.Mvc;

    using DevExpress.Data;
    using DevExpress.Data.Filtering;
    using DevExpress.Data.Filtering.Exceptions;
    using DevExpress.Data.Linq;
    using DevExpress.Data.Linq.Helpers;
    using DevExpress.Web.Mvc;

    using DocumentFormat.OpenXml;
    using DocumentFormat.OpenXml.Packaging;
    using DocumentFormat.OpenXml.Spreadsheet;
    using DocumentFormat.OpenXml.Validation;

    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.AccessRules;
    using Fruitmonitoring.Infrastructure.Security;
    using Fruitmonitoring.Models;

    using log4net;

    using Languages.Export;
    using Languages.ModelData.SampleResult;

    using Color = System.Drawing.Color;

    #endregion

    /// <summary>
    /// The exporter.
    /// </summary>
    public class Exporter
    {
        #region Constants
        /// <summary>
        /// The refvalueutilname.
        /// </summary>
        private const string REFVALUEUTILNAME = "RefValueUtilizationDummy";
        #endregion

        #region Fields
        /// <summary>
        /// The error log.
        /// </summary>
        private readonly ILog errorLog = LogManager.GetLogger("ErrorLogger");
        #endregion

        #region Static Fields
        /// <summary>
        /// The exportcolumnnamesseparator.
        /// </summary>
        public static readonly string EXPORTCOLUMNNAMESSEPARATOR = "[,]";

        /// <summary>
        /// The exportfont.
        /// </summary>
        public static readonly string EXPORTFONT = "Times New Roman";

        /// <summary>
        /// The exportfontsize.
        /// </summary>
        public static readonly int EXPORTFONTSIZE = 10;

        /// <summary>
        /// The exportheaderbackgroundcolor.
        /// </summary>
        public static readonly Color EXPORTHEADERBACKGROUNDCOLOR = Color.FromArgb(222, 247, 205);

        /// <summary>
        /// The exportheaderfontcolor.
        /// </summary>
        public static readonly Color EXPORTHEADERFONTCOLOR = Color.FromArgb(0, 0, 0);

        /// <summary>
        /// Maximum excel rows for the export.
        /// </summary>
        public static readonly int MAXROWS = 10000;
        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// Converts color to hex value without the number sign (#).
        /// </summary>
        /// <param name="color">
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public static string ColorToHex(Color color)
        {
            return null;
        }

        /// <summary>
        /// Applies filter to the model.
        /// </summary>
        /// <typeparam name="T">
        /// Model class.
        /// </typeparam>
        /// <param name="model">
        /// Model which holds all data.
        /// </param>
        /// <param name="filter">
        /// Filter expression string from the gridview.
        /// </param>
        /// <returns>
        /// Iqueryable which holds only the filtered and sorted data.
        /// </returns>
        public IQueryable<T> ApplyFilter<T>(IQueryable<T> model, string filter)
        {
            return null;
        }

        /// <summary>
        /// Applies sorting to the model.
        /// </summary>
        /// <typeparam name="T">
        /// Model class.
        /// </typeparam>
        /// <param name="model">
        /// Model which holds all data.
        /// </param>
        /// <param name="sortColumnName">
        /// Column name of the sortet column.
        /// </param>
        /// <param name="sortOrder">
        /// Order type "Ascending" or "Descending".
        /// </param>
        /// <returns>
        /// Iqueryable which holds only the filtered and sorted data.
        /// </returns>
        public IQueryable<T> ApplySorting<T>(IQueryable<T> model, string sortColumnName, string sortOrder)
        {
            return null;
        }

        /// <summary>
        /// Initialize the export.
        /// </summary>
        /// <typeparam name="T">
        /// Model type.
        /// </typeparam>
        /// <param name="columnNames">
        /// Column names.
        /// </param>
        /// <param name="columnTexts">
        /// Column captions.
        /// </param>
        /// <param name="data">
        /// List which holds the model data.
        /// </param>
        /// <param name="exportType">
        /// The export Type.
        /// </param>
        /// <param name="exportOptions">
        /// The export Options.
        /// </param>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        public ActionResult StartExport<T>(string[] columnNames, string[] columnTexts, List<T> data, string exportType, ExportOptions exportOptions)
        {
            return null;
        }

        /// <summary>
        /// The write excel stream with open xml.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <param name="columnNames">
        /// The column names.
        /// </param>
        /// <param name="columnNamesTrans">
        /// The column names trans.
        /// </param>
        /// <param name="ratings">
        /// The ratings.
        /// </param>
        /// <param name="user">
        /// The user.
        /// </param>
        /// <returns>
        /// The <see cref="MemoryStream"/>.
        /// </returns>
        public MemoryStream WriteExcelStreamWithOpenXml(
            ObjectResult<USP_Sample_Sel_BigWithResults_Result> model,
            string columnNames,
            string columnNamesTrans,
            List<USP_Rating_Sel_Result> ratings,
            CustomMembershipUser user)
        {
            return null;
        }
        #endregion

        #region Methods
        /// <summary>
        /// The get all rating areas.
        /// </summary>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_RatingArea_Sel_Result> getAllRatingAreas()
        {
            return null;
        }

        /// <summary>
        /// The add additional columns to list.
        /// </summary>
        /// <param name="columnNamesList">
        /// The column names list.
        /// </param>
        /// <param name="columnNamesTransList">
        /// The column names trans list.
        /// </param>
        /// <param name="user">
        /// The user.
        /// </param>
        /// <param name="sampleRatingPropertyNamesArray">
        /// The sample rating property names array.
        /// </param>
        /// <param name="edekaSampleRatingsPropertyNamesArray">
        /// The edeka sample ratings property names array.
        /// </param>
        private void AddAdditionalColumnsToList(
            List<string> columnNamesList,
            List<string> columnNamesTransList,
            CustomMembershipUser user,
            out string[] sampleRatingPropertyNamesArray,
            out string[] edekaSampleRatingsPropertyNamesArray)
        {
            sampleRatingPropertyNamesArray = new string[2];
            edekaSampleRatingsPropertyNamesArray = new string[2];
        }

        /// <summary>
        /// Adds lower function to the flter criteria if the comparison is based on strings.
        /// </summary>
        /// <param name="criteriaOperator">
        /// The criteria operator.
        /// </param>
        /// <returns>
        /// The filter string with added lower() functions.
        /// </returns>
        private string addLower(CriteriaOperator criteriaOperator)
        {
            return null;
        }

        /// <summary>
        /// Generate the gridview settings for the export.
        /// </summary>
        /// <param name="columnNames">
        /// An string array that holds all table column names.
        /// </param>
        /// <param name="columnTexts">
        /// An string array that holds the real names (translated).
        /// </param>
        /// <param name="exportOptions">
        /// Export options that are used by this function an set by the controller. can be used to customize the export.
        /// </param>
        /// <param name="modelType">
        /// The type of the model that is exportet.
        /// </param>
        /// <returns>
        /// Gridview settings.
        /// </returns>
        private GridViewSettings CreateGridViewSettings(string[] columnNames, string[] columnTexts, ExportOptions exportOptions, Type modelType)
        {
            return null;
        }

        /// <summary>
        /// The get edeka sample ratings property names array.
        /// </summary>
        /// <returns>
        /// The <see cref="string[]"/>.
        /// </returns>
        private string[] getEdekaSampleRatingsPropertyNamesArray()
        {
            return new[]
                   {
                       CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C22_EDEKA_PesticidesExceedingMaximum_),
                       CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C23_EDEKA_PesticidesNumberIngredients),
                       CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C24_EDEKA_AuthenticityVariety)
                   };
        }

        /// <summary>
        /// The get edeka sample ratings property names trans array.
        /// </summary>
        /// <param name="ratingAreas">
        /// The rating areas.
        /// </param>
        /// <returns>
        /// The <see cref="string[]"/>.
        /// </returns>
        private string[] getEdekaSampleRatingsPropertyNamesTransArray(List<USP_RatingArea_Sel_Result> ratingAreas)
        {
            var qualifier = "(Edeka) ";
            return new[]
                   {
                       qualifier + getRatingAreaNameTrans(ratingAreas, "46CDE011-9E21-4769-81B3-9DFBA86791F0"),

                       // pesticides: exceeding maximum residue levels
                       qualifier + getRatingAreaNameTrans(ratingAreas, "689CBF43-115E-46EF-BE17-040F09A454F4"),

                       // pesticides: number of active ingredients
                       qualifier + getRatingAreaNameTrans(ratingAreas, "4768EA80-E683-4F69-A938-94D5AF6ADF2D"), // authenticity of variety
                   };
        }

        /// <summary>
        /// Get the needed exporter.
        /// </summary>
        /// <param name="exportType">
        /// Given export type as string (e.g. "XLSX" or "PDF".
        /// </param>
        /// <returns>
        /// The <see cref="IDataExport"/>.
        /// </returns>
        private IDataExport getExportObject(string exportType)
        {
            return null;
        }

        /// <summary>
        /// The get rating area name trans.
        /// </summary>
        /// <param name="ratingAreas">
        /// The rating areas.
        /// </param>
        /// <param name="ratingAreaId">
        /// The rating area id.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private string getRatingAreaNameTrans(List<USP_RatingArea_Sel_Result> ratingAreas, string ratingAreaId)
        {
            return null;
        }

        /// <summary>
        /// The get sample ratings property names array.
        /// </summary>
        /// <returns>
        /// The <see cref="string[]"/>.
        /// </returns>
        private string[] getSampleRatingsPropertyNamesArray()
        {
            var sampleRatingsPropertyNamesArray = new[]
                                                  {
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C01_nitrate),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C02_ForeignMatter),
                                                      CustomMetadataProvider.GetMemberName(
                                                          () => EFHelper.Usp_Sample_BigWithResults.C03_PesticidesNotApprovedActiveIngredient),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C04_OrganicProduct),
                                                      CustomMetadataProvider.GetMemberName(
                                                          () => EFHelper.Usp_Sample_BigWithResults.C05_OpticalMicroscopicFindings),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C06_Radioactivity),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C07_RadioactiveIrradiation),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C08_ApprovedGMO),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C09_NotApprovedGMO),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C10_Microbiology),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C11_SulfurDioxideSO2),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C12_AuthenticityVariety),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C13_Marketability),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C14_Mycotoxins),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C15_HeavyMetals),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C16_Labelling),
                                                      CustomMetadataProvider.GetMemberName(
                                                          () => EFHelper.Usp_Sample_BigWithResults.C17_UnwantedActiveIngredients),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C18_PesticidesNumber),
                                                      CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample_BigWithResults.C19_PesticidesPercentageADI),
                                                      CustomMetadataProvider.GetMemberName(
                                                          () => EFHelper.Usp_Sample_BigWithResults.C20_PesticidesPercentageARfD),
                                                      CustomMetadataProvider.GetMemberName(
                                                          () => EFHelper.Usp_Sample_BigWithResults.C21_PesticidesExceedingMaximumResidueLevels)
                                                  };
            return sampleRatingsPropertyNamesArray;
        }

        /// <summary>
        /// The get sample ratings property names trans array.
        /// </summary>
        /// <param name="ratingAreas">
        /// The rating areas.
        /// </param>
        /// <returns>
        /// The <see cref="string[]"/>.
        /// </returns>
        private string[] getSampleRatingsPropertyNamesTransArray(List<USP_RatingArea_Sel_Result> ratingAreas)
        {
            var sampleRatingsPropertyNamesTransArray = new[]
                                                       {
                                                           getRatingAreaNameTrans(ratingAreas, "BB6C844E-33FA-4EB5-8446-CBD2BFFF503C"),
                                                           getRatingAreaNameTrans(ratingAreas, "823444B8-D955-40D4-9EEE-C768B28446D6"),
                                                           getRatingAreaNameTrans(ratingAreas, "85CB75B9-C839-460A-B595-3C01F8DD17C1"),
                                                           getRatingAreaNameTrans(ratingAreas, "DCEE6F46-9A03-412E-8CD6-3888848A192F"),
                                                           getRatingAreaNameTrans(ratingAreas, "8EE3EC9E-4EB1-4C31-A66D-AA8C76D749D1"),
                                                           getRatingAreaNameTrans(ratingAreas, "B8306376-C8AF-46DA-AD1D-3A0DA9660DFE"),
                                                           getRatingAreaNameTrans(ratingAreas, "426B4DE4-15B8-4507-B7FD-3DB51AA4211E"),
                                                           getRatingAreaNameTrans(ratingAreas, "26274A81-A519-495E-91B2-BCC6A82C0227"),
                                                           getRatingAreaNameTrans(ratingAreas, "84C3DA95-9AF6-43A6-A6A3-944370C2FE6C"),
                                                           getRatingAreaNameTrans(ratingAreas, "5665480B-2FD5-4D5C-ABEE-60F7F6C2FAEA"),
                                                           getRatingAreaNameTrans(ratingAreas, "583C4A71-6D5F-434D-9498-7E8865F3C21B"),
                                                           getRatingAreaNameTrans(ratingAreas, "D3EAC78E-FEE3-48A0-BCD9-60DAD2C11D38"),
                                                           getRatingAreaNameTrans(ratingAreas, "59FAF997-F6F3-4B39-B00D-A3F40ABDF39F"),
                                                           getRatingAreaNameTrans(ratingAreas, "44A96079-1ADC-4670-8A98-EF4F1315081E"),
                                                           getRatingAreaNameTrans(ratingAreas, "B7D12CAD-D3C7-483F-91FB-67D976898F9B"),
                                                           getRatingAreaNameTrans(ratingAreas, "82694FBA-F811-4E41-B3D5-7B14D0FC592E"),
                                                           getRatingAreaNameTrans(ratingAreas, "E8E4DE71-7536-421F-B778-59E0C790BD90"),
                                                           getRatingAreaNameTrans(ratingAreas, "052FD83A-2B59-4030-9E30-D68B415C931C"),
                                                           getRatingAreaNameTrans(ratingAreas, "4534845A-6002-43B0-ABFA-CC139869FB52"),
                                                           getRatingAreaNameTrans(ratingAreas, "93D98257-AB95-4396-98FB-7B8D0A62E655"),
                                                           getRatingAreaNameTrans(ratingAreas, "0BD528EB-3360-4342-A449-35DF51DADD9E")
                                                       };
            return sampleRatingsPropertyNamesTransArray;
        }

        /// <summary>
        /// The get sample result property names array.
        /// </summary>
        /// <returns>
        /// The <see cref="string[]"/>.
        /// </returns>
        private string[] getSampleResultPropertyNamesArray()
        {
            var sampleResultPropertyNamesArray = new[]
                                                 {
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResults_MainJoin_Parameter_Name_Trans),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResults_Included_MeasuredValuePredicate),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResults_Included_MeasuredValue),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResults_MainJoin_WeightUnit_Name_Trans),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResults_MainJoin_MeasurementPresetting_Name_Trans),                                                  
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResults_Included_CountIngredients),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResults_Included_RefValue),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResults_Included_RefValueUtilisation),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResultsSum_RefValueSum_ViaPKColumn_Sample_id),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResults_Included_ARfDValue),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResults_Included_ARfDValueUtilisation),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResultsSum_ARfDValueSum_ViaPKColumn_Sample_id),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.Parameter_Included_Undesirable),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResults_Included_ParameterType_Name_Trans),
                                                     CustomMetadataProvider.GetMemberName(
                                                         () => EFHelper.Usp_Sample_BigWithResults.SampleResults_MainJoin_EvaluationBasis_Name_Trans)
                                                 };
            return sampleResultPropertyNamesArray;
        }

        /// <summary>
        /// The get sample result property names trans array.
        /// </summary>
        /// <returns>
        /// The <see cref="string[]"/>.
        /// </returns>
        private string[] getSampleResultPropertyNamesTransArray()
        {
            var sampleResultPropertyNamesTransArray = new[]
                                                      {
                                                          SampleResult.Parameter,
                                                          SampleResult.MeasuredValuePredicate,
                                                          SampleResult.MeasuredValue,
                                                          SampleResult.WeightUnit,
                                                          SampleResult.MeasuredValueWithMeasurementpresetting,                                                        
                                                          SampleResult.CountIngredients,
                                                          SampleResult.RefValue,
                                                          SampleResult.RefValueUtilisation,
                                                          SampleResult.RefValueUtilisationSum,
                                                          SampleResult.ARfDValue,
                                                          SampleResult.ARfDValueUtilisation,
                                                          SampleResult.ARfDValueUtilisationSum,
                                                          SampleResult.ParamerterUndesirable,
                                                          SampleResult.ParameterType,
                                                          SampleResult.EvaluationBasis
                                                      };
            return sampleResultPropertyNamesTransArray;
        }

        /// <summary>
        /// Handle sample rating.
        /// </summary>
        /// <param name="ratings">
        /// The ratings.
        /// </param>
        /// <param name="value">
        /// The value.
        /// </param>
        /// <returns>
        /// The <see cref="object"/>.
        /// </returns>
        private object handleSampleRating(List<USP_Rating_Sel_Result> ratings, object value)
        {
            return null;
        }

        /// <summary>
        /// Lowercase all filter criterias.
        /// </summary>
        /// <param name="filter">
        /// Current filter string.
        /// </param>
        /// <returns>
        /// Filter string with included lowercase function (e.g. "StartsWith([SomeProperty], 'val')"  --&gt;  "StartsWith(lower([SomeProperty]), lower('val'))".
        /// </returns>
        private string parseFilterToLowerCase(string filter)
        {
            return null;
        }

        /// <summary>
        /// Set width of columns.
        /// </summary>
        /// <param name="writer">
        /// </param>
        /// <param name="numberOfColumns">
        /// </param>
        /// <param name="columnWidth">
        /// </param>
        private void setExcelColumnWidth(OpenXmlWriter writer, int numberOfColumns, int columnWidth)
        {
        }

        /// <summary>
        /// Checks spreadsheet for validity.
        /// </summary>
        /// <param name="spreadsheetDoc">
        /// </param>
        /// <returns>
        /// List of errors.
        /// </returns>
        private List<ValidationErrorInfo> validateSpreadsheet(SpreadsheetDocument spreadsheetDoc)
        {
            return null;
        }

        /// <summary>
        /// Write all column names and close row.
        /// </summary>
        /// <param name="FRUITHEADERSTYLE">
        /// The FRUITHEADERSTYLE.
        /// </param>
        /// <param name="writer">
        /// The writer.
        /// </param>
        /// <param name="columnNamesTransList">
        /// The column Names Trans List.
        /// </param>
        private void writeColumnNames(uint FRUITHEADERSTYLE, OpenXmlWriter writer, List<string> columnNamesTransList)
        {
        }

        /// <summary>
        /// Write value to cell.
        /// </summary>
        /// <param name="writer">
        /// </param>
        /// <param name="cellFormat">
        /// </param>
        /// <param name="value">
        /// </param>
        private void writeToCell(OpenXmlWriter writer, uint cellFormat, object value)
        {
        }
        #endregion
    }
}