﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="FileUtilities.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.FileHelper
{
    #region usings
    using System;
    using System.Configuration;
    using System.IO;
    using System.Web;
    using System.Web.Hosting;

    using log4net;

    using Languages;

    #endregion

    /// <summary>
    /// The file utilities.
    /// </summary>
    public class FileUtilities
    {
        #region Properties

        #region Fields
        /// <summary>
        /// The server.
        /// </summary>
        private readonly HttpServerUtilityBase server;
        #endregion

        #endregion

        #region Constructors and Destructors
        /// <summary>
        /// Initializes static members of the <see cref="FileUtilities"/> class.
        /// </summary>
        /// <exception cref="HttpException">
        /// </exception>
        static FileUtilities()
        {
            try
            {
                // ToString is needed here so it throws an error if the settings can't be found.
                DOCUMENTPATH = ConfigurationManager.AppSettings["FileUploadFolder"].ToString();
                COMMONDOCUMENTSFOLDER = ConfigurationManager.AppSettings["CommonDocumentsFolder"].ToString();
            }
            catch (NullReferenceException ex)
            {
                errorLog.ErrorFormat(ex.Message, ex.StackTrace);
                throw new HttpException(Global.GeneralError);
            }

            ;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FileUtilities"/> class.
        /// </summary>
        /// <param name="serverParam">
        /// The server param.
        /// </param>
        public FileUtilities(HttpServerUtilityBase serverParam)
        {
            server = serverParam;
        }
        #endregion

        #region Static Fields
        /// <summary>
        /// The commondocumentsfolder.
        /// </summary>
        public static string COMMONDOCUMENTSFOLDER;

        /// <summary>
        /// The documentpath.
        /// </summary>
        public static string DOCUMENTPATH;

        /// <summary>
        /// The error log.
        /// </summary>
        protected static readonly ILog errorLog = LogManager.GetLogger("ErrorLogger");
        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// Delete a file.
        /// </summary>
        /// <param name="absolutePath">
        /// Absolute file path with file name.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool DeleteFile(string absolutePath)
        {
            return false;
        }

        /// <summary>
        /// Get complete absolute path for attachments for a specific sample.
        /// </summary>
        /// <param name="date">
        /// Samples creation date.
        /// </param>
        /// <param name="sampleId">
        /// Sample id.
        /// </param>
        /// <param name="companyId">
        /// Company lab id.
        /// </param>
        /// <param name="createIfNotExists">
        /// The create If Not Exists.
        /// </param>
        /// <returns>
        /// An absolute path (e.g. "2013/02/13/" + "companyId/sampleId/").
        /// </returns>
        public string GetAbsolutePath(DateTime date, string sampleId, Guid companyId, bool createIfNotExists = true)
        {
            return null;
        }

        /// <summary>
        /// Get complete absolute path for common documents.
        /// </summary>
        /// <param name="commonDocumentsFolderName">
        /// The common Documents Folder Name.
        /// </param>
        /// <param name="companyId">
        /// Company lab id.
        /// </param>
        /// <param name="createIfNotExists">
        /// The create If Not Exists.
        /// </param>
        /// <returns>
        /// An absolute path (e.g. "2013/02/13/" + "companyId/sampleId/").
        /// </returns>
        public string GetAbsolutePath(string commonDocumentsFolderName, Guid companyId, bool createIfNotExists = true)
        {
            return null;
        }

        /// <summary>
        /// Get a absolute path.
        /// </summary>
        /// <param name="dirPath">
        /// Dir name.
        /// </param>
        /// <param name="fileName">
        /// File name.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string GetAbsolutePath(string dirPath, string fileName)
        {
            return null;
        }

        /// <summary>
        /// Gets the doument path
        /// a relative path without the main folder.
        /// </summary>
        /// <param name="date">
        /// The date.
        /// </param>
        /// <param name="sampleId">
        /// The sample Id.
        /// </param>
        /// <param name="companyId">
        /// The company Id.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string GetAttachmentPath(DateTime date, string sampleId, Guid companyId)
        {
            return null;
        }

        /// <summary>
        /// Gets the common documents path.
        /// A relative path without the main folder.
        /// </summary>
        /// <param name="commonDocumentsFolderName">
        /// The common Documents Folder Name.
        /// </param>
        /// <param name="companyId">
        /// The company Id.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string GetAttachmentPath(string commonDocumentsFolderName, Guid companyId)
        {
            return null;
        }

        /// <summary>
        /// Get the files relative path.
        /// </summary>
        /// <param name="date">
        /// Samples creation date.
        /// </param>
        /// <param name="sampleId">
        /// Sample id.
        /// </param>
        /// <param name="companyId">
        /// Company lab id.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string GetRelativePath(DateTime date, string sampleId, Guid companyId)
        {
            return null;
        }

        /// <summary>
        /// Get relative path for common documents.
        /// </summary>
        /// <param name="commonDocumentsFolderName">
        /// The common Documents Folder Name.
        /// </param>
        /// <param name="companyId">
        /// The company Id.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string GetRelativePath(string commonDocumentsFolderName, Guid companyId)
        {
            return null;
        }

        /// <summary>
        /// Get the files relative path.
        /// </summary>
        /// <param name="filePath">
        /// Relative path.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string GetRelativePath(string filePath)
        {
            return null;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Checks if directory exists and creates it (if createIfNotExists is true).
        /// </summary>
        /// <param name="createIfNotExists">
        /// </param>
        /// <param name="relPath">
        /// </param>
        /// <param name="absolutePath">
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private static string CheckIfDirectoryExists(bool createIfNotExists, string relPath, string absolutePath)
        {
            return null;
        }

        /// <summary>
        /// Gets a directory path by a given date. if the given date is 30.01.2013 this function returns the path "2013/01/13".
        /// </summary>
        /// <param name="date">
        /// Date.
        /// </param>
        /// <returns>
        /// An absolute path.
        /// </returns>
        private string getDatePath(DateTime date)
        {
            // build path from date
            return null;
        }

        /// <summary>
        /// Get complete absolute path.
        /// </summary>
        /// <param name="dirPath">
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private string GetDirectoryWithMapPath(string dirPath)
        {
            return null;
        }
        #endregion
    }
}