﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="ExtensionsFactory.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.HtmlExtensions
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Web.Mvc;

    using Fruitmonitoring.Helper.HtmlExtensions.AdvancedSearch;
    using Fruitmonitoring.Helper.HtmlExtensions.BackButton;
    using Fruitmonitoring.Helper.HtmlExtensions.BrowserInformationSendButton;
    using Fruitmonitoring.Helper.HtmlExtensions.CommonPopup;
    using Fruitmonitoring.Helper.HtmlExtensions.CommonPopupDeactivateJSValidation;
    using Fruitmonitoring.Helper.HtmlExtensions.CommonPopupNewButton;
    using Fruitmonitoring.Helper.HtmlExtensions.CommonPopupSubmitButton;
    using Fruitmonitoring.Helper.HtmlExtensions.CompanySearch;
    using Fruitmonitoring.Helper.HtmlExtensions.CompanySelection;
    using Fruitmonitoring.Helper.HtmlExtensions.CopyFieldValuesButton;
    using Fruitmonitoring.Helper.HtmlExtensions.CustomizationWindowButton;
    using Fruitmonitoring.Helper.HtmlExtensions.DocumentFileUploader;
    using Fruitmonitoring.Helper.HtmlExtensions.DropDownEditSelection;
    using Fruitmonitoring.Helper.HtmlExtensions.ExportButton;
    using Fruitmonitoring.Helper.HtmlExtensions.InvestigationParameterManagement;
    using Fruitmonitoring.Helper.HtmlExtensions.LaboratoryMethodManagement;
    using Fruitmonitoring.Helper.HtmlExtensions.LanguageSwitcher;
    using Fruitmonitoring.Helper.HtmlExtensions.PeriodOfTimeSelection;
    using Fruitmonitoring.Helper.HtmlExtensions.PopupButton;
    using Fruitmonitoring.Helper.HtmlExtensions.PopupWrapper;
    using Fruitmonitoring.Helper.HtmlExtensions.PrintButton;
    using Fruitmonitoring.Helper.HtmlExtensions.ProductSelection;
    using Fruitmonitoring.Helper.HtmlExtensions.ProductSingleElementsManagement;
    using Fruitmonitoring.Helper.HtmlExtensions.ResetLayoutButton;
    using Fruitmonitoring.Helper.HtmlExtensions.SamplerUserManagement;
    using Fruitmonitoring.Helper.HtmlExtensions.SamplerUserSearch;
    using Fruitmonitoring.Helper.HtmlExtensions.SampleRatingManagement;
    using Fruitmonitoring.Helper.HtmlExtensions.SampleResultsManagement;
    using Fruitmonitoring.Helper.HtmlExtensions.ShowActionsButton;
    using Fruitmonitoring.Helper.HtmlExtensions.SortRowsHandler;
    using Fruitmonitoring.Helper.HtmlExtensions.TranslationCreator;
    using Fruitmonitoring.Helper.HtmlExtensions.TranslationCreatorPopUp;
    using Fruitmonitoring.Helper.HtmlExtensions.Widgets;
    using Fruitmonitoring.Helper.Utility;

    #endregion

    /// <summary>
    /// This class provides access to the fruitmonitoring extensions to render components for the view.
    /// </summary>
    public class ExtensionsFactory
    {
        #region PopupButtonType enum

        #region Enums
        /// <summary>
        /// The popup button type.
        /// </summary>
        public enum PopupButtonType
        {
            /// <summary>
            /// The gridviewsave.
            /// </summary>
            GRIDVIEWSAVE,

            /// <summary>
            /// The gridviewcancel.
            /// </summary>
            GRIDVIEWCANCEL,

            /// <summary>
            /// The popupclose.
            /// </summary>
            POPUPCLOSE,

            /// <summary>
            /// The popupprint.
            /// </summary>
            POPUPPRINT
        }
        #endregion

        #endregion

        #region Properties
        // current htmlHelper Object to get access to some context data
        /// <summary>
        /// Gets or sets the html helper.
        /// </summary>
        private HtmlHelper htmlHelper { get; set; }

        /// <summary>
        /// Gets the all areas path.
        /// </summary>
        public static string AllAreasPath
        {
            get
            {
                return "Fruitmonitoring/AllAreas/";
            }
        }

        /// <summary>
        /// Gets the components path.
        /// </summary>
        public static string ComponentsPath
        {
            get
            {
                return "Fruitmonitoring/Components/";
            }
        }

        /// <summary>
        /// Gets the email template path.
        /// </summary>
        public static string EmailTemplatePath
        {
            get
            {
                return "Fruitmonitoring/Emails/";
            }
        }
        #endregion

        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="ExtensionsFactory"/> class.
        /// </summary>
        /// <param name="htmlHelper">
        /// The html helper.
        /// </param>
        public ExtensionsFactory(HtmlHelper htmlHelper)
        {
        }
        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// Builds the "AdvancedSearch".
        /// </summary>
        /// <param name="handlerClassType">
        /// Type of class that handles the users search input.
        /// </param>
        /// <param name="gridViewId">
        /// Gridview that show the search results.
        /// </param>
        /// <returns>
        /// The "SampleAdvancedSearch" html.
        /// </returns>
        public MvcHtmlString AdvancedSearch(Type handlerClassType, string gridViewId)
        {
            return null;
        }

        /// <summary>
        /// Render the "back" button.
        /// </summary>
        /// <param name="controller">
        /// Controller for the action.
        /// </param>
        /// <param name="action">
        /// Action which handles view for "back".
        /// </param>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString BackButton(string controller, string action)
        {
            return null;
        }

        /// <summary>
        /// Builds the "BrowserInformation".
        /// </summary>
        /// <returns>
        /// The "BrowserInformation" html.
        /// </returns>
        public MvcHtmlString BrowserInformation()
        {
            return null;
        }

        /// <summary>
        /// Builds the "BrowserInformationSendButton".
        /// </summary>
        /// <returns>
        /// The "BrowserInformationSendButton" html.
        /// </returns>
        public MvcHtmlString BrowserInformationSendButton()
        {
            return null;
        }

        /// <summary>
        /// Builds the "CommonPopup".
        /// </summary>
        /// <param name="additionalPartialAddFormViewPath">
        /// Path to an partial view that is rendered to the bottom button bar in add form.
        /// </param>
        /// <param name="additionalPartialEditFormViewPath">
        /// Path to an partial view that is rendered to the bottom button bar in edit form.
        /// </param>
        /// <param name="showDefaultSubmitButton">
        /// Shows the default submit button.
        /// </param>
        /// <returns>
        /// The "CommonPopup" html.
        /// </returns>
        public MvcHtmlString CommonPopup(
            string additionalPartialAddFormViewPath = "",
            string additionalPartialEditFormViewPath = "",
            bool showDefaultSubmitButton = true)
        {
            return null;
        }

        /// <summary>
        /// Renders a hidden field which can be used, to deactivate javascript validation.
        /// </summary>
        /// <returns>
        /// The "deactivate js validation" html.
        /// </returns>
        public MvcHtmlString CommonPopupDeactivateJSValidation()
        {
            return null;
        }

        /// <summary>
        /// Render the CommonPopupNewButton.
        /// </summary>
        /// <param name="controller">
        /// Controller that hold the action.
        /// </param>
        /// <param name="action">
        /// Action in the controller that renders the partial view.
        /// </param>
        /// <param name="headerText">
        /// Popup header text.
        /// </param>
        /// <param name="gridViewId">
        /// Active gridView id.
        /// </param>
        /// <param name="buttonText">
        /// Set button text. if this value is not provided, an default text is set.
        /// </param>
        /// <param name="showDefaultSubmitButton">
        /// Show the default submitbutton?.
        /// </param>
        /// <param name="showAdditionalContentInPopup">
        /// Show or hide the additional content container.
        /// </param>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString CommonPopupNewButton(
            string controller,
            string action,
            string headerText,
            string gridViewId = "",
            string buttonText = "",
            bool showDefaultSubmitButton = true,
            bool showAdditionalContentInPopup = true)
        {
            return null;
        }

        /// <summary>
        /// Render the "CommonPopupSubmitButton" button.
        /// </summary>
        /// <param name="buttonText">
        /// Buttons caption. provide an emtpy string to get default caption.
        /// </param>
        /// <param name="callbackFunctionStart">
        /// The callback Function Start.
        /// </param>
        /// <param name="callbackFunctionSucceed">
        /// A javascript callback on ajax succeed.
        /// </param>
        /// <param name="callbackFunctionError">
        /// A javascript callback on js validation failed.
        /// </param>
        /// <param name="buttonName">
        /// Unique identifier for the button.
        /// </param>
        /// <param name="routeData">
        /// Route data that should send to the server.
        /// </param>
        /// <param name="validateForm">
        /// Deactivate client side validation.
        /// </param>
        /// <param name="customActionURL">
        /// Provide a custom action url (different from forms action url).
        /// </param>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString CommonPopupSubmitButton(
            string buttonText,
            string callbackFunctionStart = "",
            string callbackFunctionSucceed = "",
            string callbackFunctionError = "",
            string buttonName = "btnSubmit",
            object routeData = null,
            bool validateForm = true,
            string customActionURL = "")
        {
            return null;
        }

        /// <summary>
        /// Builds the "CompanySearch" popup.
        /// </summary>
        /// <param name="modelPropertyName">
        /// Name of the model property, that should bind to the company selection.
        /// </param>
        /// <param name="modelPropertyValue">
        /// Value of the model proterty.
        /// </param>
        /// <param name="companyTypes">
        /// Company types, that should be show in the gridview.
        /// </param>
        /// <param name="buttonEnabled">
        /// The button Enabled.
        /// </param>
        /// <returns>
        /// The "company search" button + popup html.
        /// </returns>
        public MvcHtmlString CompanySearch(
            string modelPropertyName,
            string modelPropertyValue,
            List<DbGuidHelper.CompanyTypes> companyTypes = null,
            bool buttonEnabled = true)
        {
            return null;
        }

        /// <summary>
        /// Builds the "CompanySelection" popup.
        /// </summary>
        /// <param name="modelPropertyName">
        /// Name of the model property, that should bind to the company selection.
        /// </param>
        /// <param name="modelPropertyValue">
        /// Value of the model proterty.
        /// </param>
        /// <param name="companyTypes">
        /// Company types, that should be show in the gridview.
        /// </param>
        /// <param name="canAdministrate">
        /// User can add new companies?.
        /// </param>
        /// <param name="guidFilterComponentName">
        /// The guid Filter Component Name.
        /// </param>
        /// <param name="onlyActiveCompanies">
        /// Show only active companies.
        /// </param>
        /// <returns>
        /// The "CompanySelection" popup html.
        /// </returns>
        public MvcHtmlString CompanySelection(
            string modelPropertyName,
            string modelPropertyValue,
            List<DbGuidHelper.CompanyTypes> companyTypes = null,
            bool canAdministrate = false,
            string guidFilterComponentName = null,
            bool onlyActiveCompanies = true)
        {
            return null;
        }

        /// <summary>
        /// Builds the "CopyFieldValuesButton".
        /// </summary>
        /// <param name="buttonName">
        /// Unique name of the button.
        /// </param>
        /// <param name="buttonText">
        /// Caption of the button.
        /// </param>
        /// <param name="sourceFieldNames">
        /// Field ids that holds the source strings that should be copied.
        /// </param>
        /// <param name="targetFieldNames">
        /// Ids of all fields, that should receive the copied texts.
        /// </param>
        /// <param name="jsFunction">
        /// Provide an additional immediate javascript function that is called, if the button is clicked. (syntax: '(function(calledFunction();){}())'.
        /// </param>
        /// <param name="enabled">
        /// The enabled.
        /// </param>
        /// <returns>
        /// The "CopyFieldValuesButton" html.
        /// </returns>
        public MvcHtmlString CopyFieldValuesButton(
            string buttonName,
            string buttonText,
            List<string> sourceFieldNames,
            List<string> targetFieldNames,
            string jsFunction = "",
            bool enabled = true)
        {
            return null;
        }

        /// <summary>
        /// Render the "customization window" button.
        /// </summary>
        /// <param name="gridViewId">
        /// Active gridview retialerCompanyId.
        /// </param>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString CustomizationWindowButton(string gridViewId)
        {
            return null;
        }

        /// <summary>
        /// Builds the "DocumentFileUploader".
        /// </summary>
        /// <param name="sampleId">
        /// Sample retialerCompanyId.
        /// </param>
        /// <param name="companyId">
        /// Current users company id.
        /// </param>
        /// <param name="showDeleteButton">
        /// Show delete button in gridview?.
        /// </param>
        /// <param name="autoResizeGridview">
        /// The auto Resize Gridview.
        /// </param>
        /// <returns>
        /// The "DocumentFileUploader" html.
        /// </returns>
        public MvcHtmlString DocumentFileUploader(Guid? sampleId, Guid? companyId, bool showDeleteButton, bool autoResizeGridview = false)
        {
            return null;
        }

        /// <summary>
        /// Builds the "DropDownEditSelection".
        /// </summary>
        /// <param name="componentName">
        /// Name of the component.
        /// </param>
        /// <param name="data">
        /// Data for the dropdown.
        /// </param>
        /// <param name="nameProperty">
        /// Member in the data which should be set as the dropdown-name.
        /// </param>
        /// <param name="valueProperty">
        /// Member in the data which should be set as the dropdown-value.
        /// </param>
        /// <param name="dataPreSelectionIds">
        /// Element ids that should be preselected.
        /// </param>
        /// <param name="namePropertyPreSelection">
        /// Name of the property that holds the preselection value.
        /// </param>
        /// <returns>
        /// The "DropDownEditSelection" html.
        /// </returns>
        public MvcHtmlString DropDownEditSelection<E>(
            string componentName,
            List<E> data,
            string nameProperty,
            string valueProperty,
            List<Guid> dataPreSelectionIds,
            string namePropertyPreSelection)
        {
            return null;
        }

        /// <summary>
        /// Render the "export" button.
        /// </summary>
        /// <param name="controllerType">
        /// Type of the controller, which should handle the request.
        /// </param>
        /// <param name="gridViewId">
        /// Active gridview retialerCompanyId.
        /// </param>
        /// <param name="exportAction">
        /// The export Action.
        /// </param>
        /// <param name="buttonTitle">
        /// The button Title.
        /// </param>
        /// <param name="enableFilterOption">
        /// Enables filter option.
        /// </param>
        /// <param name="enableSortingOption">
        /// Enables sorting option.
        /// </param>
        /// <param name="jsVarsExportParams">
        /// Additional parameters that will be added to export.
        /// Provide names of JavaScript variables that will be read to runtime.
        /// </param>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString ExportButton(
            Type controllerType,
            string gridViewId,
            string exportAction = null,
            string buttonTitle = null,
            bool enableFilterOption = true,
            bool enableSortingOption = true,
            List<string> jsVarsExportParams = null)
        {
            return null;
        }

        /// <summary>
        /// Builds the parameter selection for the investigation order form.
        /// </summary>
        /// <param name="sampleGuid">
        /// Curremt sample id.
        /// </param>
        /// <returns>
        /// The "InvestigationParameterManagement" html.
        /// </returns>
        public MvcHtmlString InvestigationParameterManagement(Guid sampleGuid)
        {
            return null;
        }

        /// <summary>
        /// Builds the "LaboratoryMethodManagement".
        /// </summary>
        /// <returns>
        /// The "LaboratoryMethodManagement" html.
        /// </returns>
        public MvcHtmlString LaboratoryMethodManagement()
        {
            return null;
        }

        /// <summary>
        /// Render the languageswitcher.
        /// </summary>
        /// <param name="settings">
        /// Settings for the language switcher.
        /// </param>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString LanguageSwitcher(Action<LanguageSwitcherSettings> settings)
        {
            return null;
        }

        /// <summary>
        /// Render the "period of time selection" button.
        /// </summary>
        /// <param name="controllerType">
        /// Type of the controller, which should handle the request.
        /// </param>
        /// <param name="gridViewId">
        /// Active gridview retialerCompanyId.
        /// </param>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString PeriodOfTimeSelection(Type controllerType, string gridViewId)
        {
            return null;
        }

        /// <summary>
        /// Builds a "PopupButton".
        /// </summary>
        /// <param name="buttonType">
        /// The functionality of the popupbutton.
        /// </param>
        /// <param name="popupName">
        /// Name of the popup.
        /// </param>
        /// <param name="gridViewId">
        /// The popups gridview id (needed for gridview-save and gridview-close).
        /// </param>
        /// <param name="formName">
        /// Form id/name that should be validate clientside.
        /// </param>
        /// <param name="popupContentContainerName">
        /// Name of the content container in the popup (needed for automatic resizing action).
        /// </param>
        /// <param name="actionUrl">
        /// The action Url.
        /// </param>
        /// <param name="fieldName">
        /// The field Name.
        /// </param>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString PopupButton(
            PopupButtonType buttonType,
            string popupName,
            string gridViewId = "",
            string formName = "",
            string popupContentContainerName = "",
            string actionUrl = null,
            string fieldName = null)
        {
            return null;
        }

        /// <summary>
        /// Start rendering the popup wrapper container.
        /// </summary>
        /// <param name="popupName">
        /// Popup name that holds this content wrapper.
        /// </param>
        /// <param name="containerId">
        /// Container id.
        /// </param>
        /// <param name="containerClass">
        /// Container class.
        /// </param>
        /// <returns>
        /// The <see cref="PopupWrapperExtension"/>.
        /// </returns>
        public PopupWrapperExtension PopupWrapper(string popupName, string containerId = "", string containerClass = "")
        {
            return null;
        }

        /// <summary>
        /// Builds the "printbutton".
        /// </summary>
        /// <param name="gridViewName">
        /// Gridview name.
        /// </param>
        /// <param name="partialViewName">
        /// Print-view name.
        /// </param>
        /// <param name="controllerName">
        /// Name of a controller that should provided to the partial view.
        /// </param>
        /// <returns>
        /// The "SamplerUserManagement" popup html.
        /// </returns>
        public MvcHtmlString PrintButton(string gridViewName, string partialViewName, string controllerName = "")
        {
            return null;
        }

        /// <summary>
        /// Builds the "CompanyProductSelectionSelection" popup.
        /// </summary>
        /// <param name="modelPropertyName">
        /// Name of the model property, that should bind to the product selection.
        /// </param>
        /// <param name="modelPropertyValue">
        /// Value of the model proterty.
        /// </param>
        /// <returns>
        /// The "ProductSelection"  html.
        /// </returns>
        public MvcHtmlString ProductSelection(string modelPropertyName, string modelPropertyValue)
        {
            return null;
        }

        /// <summary>
        /// Builds the "ProductSingleElementsManagement".
        /// </summary>
        /// <param name="sampleId">
        /// Sample retialerCompanyId.
        /// </param>
        /// <param name="isOrderForm">
        /// The is Order Form.
        /// </param>
        /// <returns>
        /// The "ProductSingleElementsManagement" html.
        /// </returns>
        public MvcHtmlString ProductSingleElementsManagement(Guid sampleId, bool isOrderForm = false)
        {
            return null;
        }

        /// <summary>
        /// Render the "reset layout" button.
        /// </summary>
        /// <param name="controller">
        /// Controller for the action.
        /// </param>
        /// <param name="action">
        /// Action which handles the "reset".
        /// </param>
        /// <param name="gridViewId">
        /// Active gridview retialerCompanyId.
        /// </param>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString ResetLayoutButton(string controller, string action, string gridViewId)
        {
            return null;
        }

        /// <summary>
        /// Builds the "SampleRatingManagement".
        /// </summary>
        /// <param name="sampleId">
        /// Sample retialerCompanyId.
        /// </param>
        /// <returns>
        /// The "SampleRatingManagement" html.
        /// </returns>
        public MvcHtmlString SampleRatingManagement(Guid sampleId)
        {
            return null;
        }

        /// <summary>
        /// Builds the "SampleResultsManagement".
        /// </summary>
        /// <param name="sampleId">
        /// Sample retialerCompanyId.
        /// </param>
        /// <returns>
        /// The "SampleResultsManagement" html.
        /// </returns>
        public MvcHtmlString SampleResultsManagement(Guid sampleId)
        {
            return null;
        }

        /// <summary>
        /// Builds the "SamplerUserManagement" popup.
        /// </summary>
        /// <param name="modelPropertyName">
        /// Name of the model property, that should bind to the SamplerUserManagement.
        /// </param>
        /// <param name="modelPropertyValue">
        /// Value of the model proterty.
        /// </param>
        /// <returns>
        /// The "SamplerUserManagement" popup html.
        /// </returns>
        public MvcHtmlString SamplerUserManagement(string modelPropertyName, string modelPropertyValue)
        {
            return null;
        }

        /// <summary>
        /// Builds the "sampleruser" popup.
        /// </summary>
        /// <param name="modelPropertyName">
        /// Name of the model property, that should bind to the sampler user selection.
        /// </param>
        /// <param name="modelPropertyValue">
        /// Value of the model proterty.
        /// </param>
        /// <returns>
        /// The "sampler user search" button + popup html.
        /// </returns>
        public MvcHtmlString SamplerUserSearch(string modelPropertyName, string modelPropertyValue)
        {
            return null;
        }

        /// <summary>
        /// Render the "show actions button" button.
        /// </summary>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString ShowActionsButton()
        {
            return null;
        }

        /// <summary>
        /// Render the "sortrows" javascript.
        /// </summary>
        /// <param name="controllerType">
        /// Type of the controller, which should handle the request.
        /// </param>
        /// <param name="gridViewId">
        /// Active gridview retialerCompanyId.
        /// </param>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString SortRowsHandler(Type controllerType, string gridViewId)
        {
            return null;
        }

        /// <summary>
        /// Render the "TranslationCreator".
        /// </summary>
        /// <param name="tableName">
        /// Db table name.
        /// </param>
        /// <param name="columnName">
        /// Column name of the table.
        /// </param>
        /// <param name="rowId">
        /// CurrentRow retialerCompanyId from the dataset (the unique retialerCompanyId).
        /// </param>
        /// <param name="useBigTextBox">
        /// If true, the component renders a textarea instead of and textbox.
        /// </param>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString TranslationCreator(string tableName, string columnName, Guid? rowId, bool useBigTextBox = false)
        {
            return null;
        }

        /// <summary>
        /// Render the "TranslationCreatorPopUp".
        /// </summary>
        /// <param name="tableName">
        /// Db table name.
        /// </param>
        /// <param name="columnName">
        /// Column name of the table.
        /// </param>
        /// <param name="rowId">
        /// CurrentRow retialerCompanyId from the dataset (the unique retialerCompanyId).
        /// </param>
        /// <param name="headerText">
        /// Headline test.
        /// </param>
        /// <param name="useBigTextBox">
        /// If true, the component renders a textarea instead of and textbox.
        /// </param>
        /// <returns>
        /// Html string.
        /// </returns>
        public MvcHtmlString TranslationCreatorPopUp(string tableName, string columnName, Guid? rowId, string headerText, bool useBigTextBox = false)
        {
            return null;
        }

        /// <summary>
        /// Rendering a WidgetButton.
        /// </summary>
        /// <param name="widgetName">
        /// Name of the Widget.
        /// </param>
        /// <param name="buttonText">
        /// Text for the button.
        /// </param>
        /// <returns>
        /// The <see cref="MvcHtmlString"/>.
        /// </returns>
        public MvcHtmlString WidgetButton(string widgetName, string buttonText)
        {
            return null;
        }
        #endregion
    }
}