﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="CultureHelper.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Utility
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Threading;

    #endregion

    /// <summary>
    /// Provides some functions for language handling
    /// more information: http://afana.me/post/aspnet-mvc-internationalization.aspx.
    /// </summary>
    public static class CultureHelper
    {
        #region Languages enum

        #region Enums
        /// <summary>
        /// The languages.
        /// </summary>
        public enum Languages
        {
            /// <summary>
            /// The german.
            /// </summary>
            GERMAN,

            /// <summary>
            /// The english.
            /// </summary>
            ENGLISH

            // SPANISH,
            // ITALIAN
        }
        #endregion

        #endregion

        #region variables

        // Include ONLY cultures you are implementing as views
        /// <summary>
        /// Gets the cultures.
        /// </summary>
        public static Dictionary<string, bool> Cultures
        {
            get
            {
                return new Dictionary<string, bool>
                       {
                           { "en-GB", true },
                           { "de-DE", false } // false indecates, that we use the default view 
                           // first culture is the DEFAULT (do not change!)
                           // {"es-ES", false},
                           // {"it-IT", false}
                       };
            }
        }

        /// <summary>
        /// Gets the cultures is o 639_3.
        /// </summary>
        public static Dictionary<string, string> CulturesISO639_3
        {
            get
            {
                return new Dictionary<string, string>
                       {
                           { "en-GB", "ENG" },
                           { "de-DE", "GER" }

                           // first culture is the DEFAULT (do not change!)
                           // {"es-ES", "ESP"},
                           // {"it-IT", "ITA"}
                       };
            }
        }
        #endregion

        #region methods
        /// <summary>
        /// Returns the valid culture name based on the given enum parameter.
        /// </summary>
        /// <param name="language">
        /// Language.
        /// </param>
        /// <returns>
        /// E.g. "en-GB".
        /// </returns>
        public static string GetValidCulture(Languages language)
        {
            return null;
        }

        /// <summary>
        /// Returns a valid culture name based on "name" parameter. If "name" is not valid, it returns the default culture "en-GB"
        /// try to match exact culture (e.g. en-GB)
        /// if it is not found, try to match the language (e.g. en)
        /// else return the default culture.
        /// </summary>
        /// <param name="name">
        /// Culture's name (e.g. en-GB).
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public static string GetValidCulture(string name)
        {
            return null;
        }

        /// <summary>
        /// Returns default culture name which is the first name decalared in "_cultures" (e.g. en-US).
        /// </summary>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public static string GetDefaultCulture()
        {
            return null;
        }

        /// <summary>
        /// Returns "true" if view is implemented separatley, and "false" if not.
        /// For example, if "es-CL" is true, then separate views must exist e.g. Index.es-cl.cshtml, About.es-cl.cshtml.
        /// </summary>
        /// <param name="name">
        /// Culture's name.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public static bool IsViewSeparate(string name)
        {
            return false;
        }

        /// <summary>
        /// Parse a given culture to ISO639-3 value.
        /// </summary>
        /// <param name="culture">
        /// E.g. "de-DE" or "de".
        /// </param>
        /// <returns>
        /// ISO639-3 language key.
        /// </returns>
        public static string GetCultureISO639_3(string culture = "")
        {
            return null;
        }

        /// <summary>
        /// Gets current culture name, which is saved in current thread.
        /// </summary>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public static string GetCurrentCulture()
        {
            return null;
        }
        #endregion
    }
}