﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="GridViewFilterHelper.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Utility
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Text.RegularExpressions;
    using System.Web;
    using System.Xml;

    using DevExpress.Data.Filtering;

    using Fruitmonitoring.Models;

    #endregion

    /// <summary>
    /// The grid view filter helper.
    /// </summary>
    public class GridViewFilterHelper
    {
        #region Public Methods and Operators
        /// <summary>
        /// Get the value for a column.
        /// </summary>
        /// <param name="xmlFilter">
        /// XmlFilter.
        /// </param>
        /// <param name="columnName">
        /// Name of the Column.
        /// </param>
        /// <returns>
        /// Value for given ColumnName.
        /// </returns>
        public static string GetColumnValueFromXMLFilter(string xmlFilter, string columnName)
        {
            return null;
        }

        /// <summary>
        /// The get column value from xml filter.
        /// </summary>
        /// <param name="xmlFilter">
        /// The xml filter.
        /// </param>
        /// <param name="modelType">
        /// The model type.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        public static List<FilterExpression> GetColumnValueFromXMLFilter(string xmlFilter, Type modelType)
        {
            return null;
        }

        /// <summary>
        /// Adds an given xml filter node to an complete xml filter string.
        /// </summary>
        /// <param name="filterItem">
        /// New filter string ( /filterItem/ ..... /filterItem/ ).
        /// </param>
        /// <param name="xml">
        /// Given xml string.
        /// </param>
        /// <returns>
        /// Complete xml string with additional filter string.
        /// </returns>
        public string AddFilterItemToXml(string filterItem, string xml)
        {
            // use string operations to combine this xml, because this is faster than parsing to xml-nodes and back
            // get root element name
            var rootNode = Regex.Match(xml, @"<[a-zA-Z]*?>");
            if (rootNode.Success)
            {
                // get root node name
                var rootNodeValue = rootNode.Value;
                rootNodeValue = rootNodeValue.Substring(1, rootNodeValue.Length - 2);

                // remove closing root tag
                xml = xml.Substring(0, xml.LastIndexOf(string.Format("</{0}>", rootNodeValue)));

                // concat xml
                xml = string.Format("{0}{1}</{2}>", xml, filterItem, rootNodeValue);

                return xml;
            }
            else
            {
                throw new HttpException("Xml root node not found");
            }
        }

        /// <summary>
        /// Parse an devexpress filter expression to xml filter item (without root element).
        /// </summary>
        /// <param name="filterExpression">
        /// The filter Expression.
        /// </param>
        /// <param name="modelType">
        /// The model Type.
        /// </param>
        /// <returns>
        /// Xml filter item for SP (without the root element).
        /// </returns>
        public string BuildFilterXmlItem(string filterExpression, Type modelType)
        {
            return null;
        }

        /// <summary>
        /// Parse an backend object to xml filter structure.
        /// </summary>
        /// <param name="filterExpression">
        /// The filter Expression.
        /// </param>
        /// <returns>
        /// Xml node which holds the filter information.
        /// </returns>
        public string BuildFilterXmlItem(FilterExpression filterExpression)
        {
            return null;
        }

        /// <summary>
        /// Build the xml for the filter-parameter for the stored procedures.
        /// </summary>
        /// <param name="filterExpressions">
        /// List that holds all filter expression objects.
        /// </param>
        /// <returns>
        /// Xml filter string.
        /// </returns>
        public string BuildXml(List<FilterExpression> filterExpressions)
        {
            return null;
        }

        /// <summary>
        /// Build the xml for the filter-parameter for the stored procedures.
        /// </summary>
        /// <param name="filterItemAsString">
        /// </param>
        /// <returns>
        /// Xml filter string.
        /// </returns>
        public string BuildXml(string filterItemAsString)
        {
            return null;
        }

        /// <summary>
        /// Get the filter condition depending on gridviews selected filter-index.
        /// </summary>
        /// <param name="index">
        /// Selected index.
        /// </param>
        /// <returns>
        /// Filter-condition for the index.
        /// </returns>
        //public AutoFilterCondition ParseInputIndexToFilterCondition( int index )
        //{
        //    var condition = AutoFilterCondition.Default;
        //    switch ( index )
        //    {
        //        case 0:
        //            condition = AutoFilterCondition.BeginsWith;
        //            break;
        //        case 1:
        //            condition = AutoFilterCondition.Contains;
        //            break;
        //        case 2:
        //            condition = AutoFilterCondition.DoesNotContain;
        //            break;
        //        case 3:
        //            condition = AutoFilterCondition.EndsWith;
        //            break;
        //        case 4:
        //            condition = AutoFilterCondition.Equals;
        //            break;
        //        case 5:
        //            condition = AutoFilterCondition.NotEqual;
        //            break;
        //        case 6:
        //            condition = AutoFilterCondition.Less;
        //            break;
        //        case 7:
        //            condition = AutoFilterCondition.LessOrEqual;
        //            break;
        //        case 8:
        //            condition = AutoFilterCondition.Greater;
        //            break;
        //        case 9:
        //            condition = AutoFilterCondition.GreaterOrEqual;
        //            break;
        //        default:
        //            condition = AutoFilterCondition.BeginsWith;
        //            break;
        //    }

        //    return condition;
        //}
        /// <summary>
        /// The escape for xml.
        /// </summary>
        /// <param name="s">
        /// The s.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string EscapeForXml(string s)
        {
            return null;
        }

        /// <summary>
        /// Parses the filter expression string, which is given from the devexpress gridview, into single parts
        /// given: e.g. "StartsWith([Sample_id], 'test') And StartsWith([Account_id], 'test2')"
        /// - "Sample_id" is the key
        /// - "test" is the value
        /// - "startswith" must be convertet to an sql expression like "value%"
        /// First of all the complete filter expression is splittet at "And"
        /// Currently there is no way to get the plain filter-strings, so we parse it here.
        /// </summary>
        /// <param name="filterExpression">
        /// The filter Expression.
        /// </param>
        /// <param name="modelType">
        /// The model Type.
        /// </param>
        /// <returns>
        /// A list fo single filter expressions which can be used to fill the xml-string for the stored procedure.
        /// </returns>
        public List<FilterExpression> FilterExpressionToList(string filterExpression, Type modelType)
        {
            return null;
        }

        /// <summary>
        /// Builds the xml filter string for the SP.
        /// </summary>
        /// <param name="filterExpr">
        /// Filter expression given from the gridview.
        /// </param>
        /// <param name="modelType">
        /// The model Type.
        /// </param>
        /// <returns>
        /// Xml filter string.
        /// </returns>
        public string GenerateXmlFilter(string filterExpr, Type modelType)
        {
            return null;
        }

        /// <summary>
        /// Get the xml filter item to filter for IN Statements.
        /// </summary>
        /// <param name="ids">
        /// The ids.
        /// </param>
        /// <param name="columnName">
        /// The column Name.
        /// </param>
        /// <param name="modelType">
        /// The model Type.
        /// </param>
        /// <returns>
        /// XML Filter for IN statements.
        /// </returns>
        public string GetFilterItemXmlForInFilterStatement(List<Guid> ids, string columnName, Type modelType)
        {
            return null;
        }

        /// <summary>
        /// get the xml filter item to filter all companies by their types.
        /// </summary>
        /// <param name="types">
        /// The types.
        /// </param>
        /// <returns>
        /// Xml filter item.
        /// </returns>
        public string GetFilterXmlForCompanyTypes(List<DbGuidHelper.CompanyTypes> types)
        {
            return null;
        }

        /// <summary>
        /// Get xml regions filter.
        /// </summary>
        /// <param name="ids">
        /// Company ids for companyref.
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string GetFilterXmlForRegions(List<Guid> ids)
        {
            return null;
        }

        /// <summary>
        /// Get the xml filter item to filter all companies by their ids.
        /// </summary>
        /// <param name="ids">
        /// The ids.
        /// </param>
        /// <param name="keyField">
        /// Table column name.
        /// </param>
        /// <param name="modelType">
        /// The model Type.
        /// </param>
        /// <returns>
        /// Xml filter item.
        /// </returns>
        public string GetXmlFilter_InArray(List<Guid> ids, string keyField, Type modelType)
        {
            return null;
        }

        /// <summary>
        /// Check for compound Date filter-expression with OR and replace with NOT operator
        /// and reduce to a single filter-expression.
        /// </summary>
        /// <param name="filterExpression">
        /// </param>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public string ReplaceNotDateFilter(string filterExpression)
        {
            return null;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Parse a single criteriaOperator-object to an filterExpression-object.
        /// </summary>
        /// <param name="criteria">
        /// The criteria.
        /// </param>
        /// <param name="modelType">
        /// The model Type.
        /// </param>
        /// <returns>
        /// Filter expression object which holds the values from the criteriaOperator operator.
        /// </returns>
        private FilterExpression buildFilterExpression(CriteriaOperator criteria, Type modelType)
        {
            return null;
        }

        /// <summary>
        /// Add parentheses at the beginning and the end of a string
        /// Replae semicolon with comma.
        /// </summary>
        /// <param name="input">
        /// Input text.
        /// </param>
        /// <returns>
        /// Input text without brackets.
        /// </returns>
        private string prepareValueFromINOperator(string input)
        {
            return null;
        }

        /// <summary>
        /// Removes brackets at the beginning and the end of a string.
        /// </summary>
        /// <param name="input">
        /// Input text.
        /// </param>
        /// <returns>
        /// Input text without brackets.
        /// </returns>
        private string removeBrackets(string input)
        {
            return null;
        }

        /// <summary>
        /// Removes quotation marks at the beginning and the end of a string.
        /// </summary>
        /// <param name="input">
        /// Input text.
        /// </param>
        /// <returns>
        /// Input text without quotation marks.
        /// </returns>
        private string removeQuotationMarks(string input)
        {
            return null;
        }
        #endregion
    }
}