﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="GuidNotEmtpyAttribute.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Validation
{
    #region usings
    using System;
    using System.ComponentModel.DataAnnotations;
    using System.Globalization;

    #endregion

    /// <summary>
    /// Check if an guid is Guid.Empty.
    /// </summary>
    public class GuidNotEmtpyAttribute : ValidationAttribute
    {
        #region Public Methods and Operators
        /// <summary>
        /// Custom error message format.
        /// </summary>
        /// <param name="name">
        /// The name.
        /// </param>
        /// <returns>
        /// Formatted error message.
        /// </returns>
        public override string FormatErrorMessage(string name)
        {
            return null;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Validate that the guid is empty.
        /// </summary>
        /// <param name="value">
        /// Current validated value.
        /// </param>
        /// <param name="validationContext">
        /// Validation context.
        /// </param>
        /// <returns>
        /// Null if validation succeed, otherwise an Validation Result.
        /// </returns>
        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            try
            {
                var guid = Guid.Parse(value.ToString());

                if (Guid.Empty.Equals(guid))
                {
                    // guid is guid.empty
                    return new ValidationResult(FormatErrorMessage(validationContext.DisplayName));
                }
                else
                {
                    return null;
                }
            }
            catch (Exception)
            {
                return new ValidationResult(FormatErrorMessage(validationContext.DisplayName));
            }
        }
        #endregion
    }
}