﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="IsNotGuidAttribute.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Validation
{
    #region usings
    using System;
    using System.ComponentModel.DataAnnotations;
    using System.Globalization;

    #endregion

    /// <summary>
    /// Check if an guid does not equal a specific value.
    /// </summary>
    public class IsNotGuidAttribute : ValidationAttribute
    {
        #region Properties
        /// <summary>
        /// Gets the compare guid.
        /// </summary>
        public string CompareGuid { get; private set; }
        #endregion

        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="IsNotGuidAttribute"/> class.
        /// </summary>
        /// <param name="compareGuid">
        /// The guid that should be compared.
        /// </param>
        public IsNotGuidAttribute(string compareGuid)
        {
            CompareGuid = compareGuid;
        }
        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// Custom error message format.
        /// </summary>
        /// <param name="name">
        /// The name.
        /// </param>
        /// <returns>
        /// Formatted error message.
        /// </returns>
        public override string FormatErrorMessage(string name)
        {
            return null;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Validate that the guid does not equal a specific value.
        /// </summary>
        /// <param name="value">
        /// Current validated value.
        /// </param>
        /// <param name="validationContext">
        /// Validation context.
        /// </param>
        /// <returns>
        /// Null if validation succeed, otherwise an Validation Result.
        /// </returns>
        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            return null;
        }
        #endregion
    }
}