#region Copyright  convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="RequiredIfNotPropertyHasValue.cs.cs" company="convivo GmbH">
//    Copyright  2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Validation
{
    /// <summary>
    /// RequiredIfNot attribute. If "propertyName" does NOT have "desiredValue", the annotated property will be required to match the value in "pattern".
    /// </summary>
    public class RequiredIfNotPropertyHasValue : RequiredIfPropertyHasValue
    {
        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="RequiredIfNotPropertyHasValue"/> class.
        /// RequiredIfNot attribute. If "propertyName" does NOT have "desiredValue", the annotated property will be required to match the value in "pattern".
        /// </summary>
        /// <param name="propertyName">
        /// Property in the model.
        /// </param>
        /// <param name="desiredValue">
        /// Value the property has.
        /// </param>
        /// <param name="pattern">
        /// The annotated property's value must match this pattern if the other property does NOT have the desired value.
        /// </param>
        /// <param name="toLower">
        /// The to Lower.
        /// </param>
        public RequiredIfNotPropertyHasValue(string propertyName, object desiredValue, string pattern, bool toLower = false)
            : base(propertyName, desiredValue, pattern, toLower)
        {
        }
        #endregion

        #region Methods
        /// <summary>
        /// The compare values.
        /// </summary>
        /// <param name="propertyValueString">
        /// The property value string.
        /// </param>
        /// <param name="desiredValueString">
        /// The desired value string.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        protected override bool CompareValues(string propertyValueString, string desiredValueString)
        {
            return false;
        }
        #endregion
    }
}