﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="FruitmonitoringRazorViewEngine.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.View
{
    #region usings
    using System.Diagnostics;
    using System.Web.Mvc;

    #endregion

    /// <summary>
    /// This class overrides the default view engine, so we can structurize the view folder in the same way as the controller folder (subfolders etc.).
    /// </summary>
    public class FruitmonitoringRazorViewEngine : RazorViewEngine
    {
        #region Constructors and Destructors

        #region constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="FruitmonitoringRazorViewEngine"/> class.
        /// </summary>
        public FruitmonitoringRazorViewEngine()
        {
            AreaViewLocationFormats = new[]
                                      {
                                          "~/Areas/{2}/Views/{1}/{0}.cshtml", "~/Areas/{2}/Views/{1}/{0}.vbhtml", "~/Areas/{2}/Views/Shared/{0}.cshtml",
                                          "~/Areas/{2}/Views/Shared/{0}.vbhtml"
                                      };

            AreaMasterLocationFormats = new[]
                                        {
                                            "~/Areas/{2}/Views/{1}/{0}.cshtml", "~/Areas/{2}/Views/{1}/{0}.vbhtml", "~/Areas/{2}/Views/Shared/{0}.cshtml",
                                            "~/Areas/{2}/Views/Shared/{0}.vbhtml"
                                        };

            AreaPartialViewLocationFormats = new[]
                                             {
                                                 "~/Areas/{2}/Views/{1}/{0}.cshtml", "~/Areas/{2}/Views/{1}/{0}.vbhtml", "~/Areas/{2}/Views/Shared/{0}.cshtml",
                                                 "~/Areas/{2}/Views/Shared/{0}.vbhtml"
                                             };

            ViewLocationFormats = new[]
                                  {
                                      "~/Views/%1/{1}/{0}.cshtml", "~/Views/%1/{1}/{0}.vbhtml", "~/Views/%1/Shared/{0}.cshtml", "~/Views/%1/Shared/{0}.vbhtml",
                                      "~/Views/{1}/{0}.cshtml", "~/Views/{1}/{0}.vbhtml", "~/Views/Shared/{0}.cshtml", "~/Views/Shared/{0}.vbhtml"
                                  };

            MasterLocationFormats = new[]
                                    {
                                        "~/Views/%1/{1}/{0}.cshtml", "~/Views/%1/{1}/{0}.vbhtml", "~/Views/%1/Shared/{0}.cshtml", "~/Views/%1/Shared/{0}.vbhtml",
                                        "~/Views/{1}/{0}.cshtml", "~/Views/{1}/{0}.vbhtml", "~/Views/Shared/{0}.cshtml", "~/Views/Shared/{0}.vbhtml"
                                    };

            PartialViewLocationFormats = new[]
                                         {
                                             "~/Views/%1/{1}/{0}.cshtml", "~/Views/%1/{1}/{0}.vbhtml", "~/Views/%1/Shared/{0}.cshtml",
                                             "~/Views/%1/Shared/{0}.vbhtml", "~/Views/{1}/{0}.cshtml", "~/Views/{1}/{0}.vbhtml", "~/Views/Shared/{0}.cshtml",
                                             "~/Views/Shared/{0}.vbhtml"
                                         };
        }
        #endregion

        #endregion

        #region methods
        /// <summary>
        /// The create partial view.
        /// </summary>
        /// <param name="controllerContext">
        /// The controller context.
        /// </param>
        /// <param name="partialPath">
        /// The partial path.
        /// </param>
        /// <returns>
        /// The <see cref="IView"/>.
        /// </returns>
        protected override IView CreatePartialView(ControllerContext controllerContext, string partialPath)
        {
            var PassedController = controllerContext.Controller;
            Debug.Assert(PassedController != null, "PassedController != null");
            return base.CreatePartialView(controllerContext, partialPath.Replace("%1", namespaceToFolder(PassedController.GetType().Namespace)));
        }

        /// <summary>
        /// The create view.
        /// </summary>
        /// <param name="controllerContext">
        /// The controller context.
        /// </param>
        /// <param name="viewPath">
        /// The view path.
        /// </param>
        /// <param name="masterPath">
        /// The master path.
        /// </param>
        /// <returns>
        /// The <see cref="IView"/>.
        /// </returns>
        protected override IView CreateView(ControllerContext controllerContext, string viewPath, string masterPath)
        {
            return null;
        }

        /// <summary>
        /// The file exists.
        /// </summary>
        /// <param name="controllerContext">
        /// The controller context.
        /// </param>
        /// <param name="virtualPath">
        /// The virtual path.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        protected override bool FileExists(ControllerContext controllerContext, string virtualPath)
        {
            return false;
        }

        /// <summary>
        /// Converts a controller namspace string to an folder string
        /// and removes the main-namespace "Fruitmonitoring.Controllers".
        /// </summary>
        /// <param name="ns">
        /// Namespace as a string.
        /// </param>
        /// <returns>
        /// Foldername as a string.
        /// </returns>
        private string namespaceToFolder(string ns)
        {
            return null;
        }
        #endregion
    }
}